@@ -20,9 +20,12 @@
 #ifndef FLAC__ALL_H
 #define FLAC__ALL_H
 
+#include "assert.h"
 #include "encoder.h"
 #include "file_decoder.h"
+#include "format.h"
 #include "ordinals.h"
+#include "seek_table.h"
 #include "stream_decoder.h"
 
 #endif
