@@ -33,7 +33,7 @@
 #define FLAC__MAX_SAMPLE_RATE (1048570u)
 #define FLAC__MAX_LPC_ORDER (32u)
 #define FLAC__MIN_QLP_COEFF_PRECISION (5u)
-/* changing this also means changing all of fixed.c and more, so DON'T! */
+/* changing FLAC__MAX_FIXED_ORDER also means changing all of fixed.c and more, so DON'T! */
 #define FLAC__MAX_FIXED_ORDER (4u)
 #define FLAC__MAX_RICE_PARTITION_ORDER (15u)
 
@@ -41,7 +41,7 @@
 #ifdef VERSION
 #define FLAC__VERSION_STRING VERSION
 #else
-#define FLAC__VERSION_STRING "0.9"
+#define FLAC__VERSION_STRING "0.10"
 #endif
 
 extern const byte     FLAC__STREAM_SYNC_STRING[4]; /* = "fLaC" */;
@@ -77,10 +77,14 @@ extern const char *FLAC__EntropyCodingMethodTypeString[];
 typedef struct {
 	unsigned order;
 	unsigned parameters[1 << FLAC__MAX_RICE_PARTITION_ORDER];
+	unsigned raw_bits[1 << FLAC__MAX_RICE_PARTITION_ORDER];
 } FLAC__EntropyCodingMethod_PartitionedRice;
 
 extern const unsigned FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_ORDER_LEN; /* = 4 bits */
 extern const unsigned FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_PARAMETER_LEN; /* = 4 bits */
+extern const unsigned FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_RAW_LEN; /* = 5 bits */
+
+extern const unsigned FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_ESCAPE_PARAMETER; /* = (1<<FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_PARAMETER_LEN)-1 */
 
 /*****************************************************************************
  *
@@ -320,7 +324,8 @@ typedef struct {
 typedef enum {
 	FLAC__METADATA_TYPE_STREAMINFO = 0,
 	FLAC__METADATA_TYPE_PADDING = 1,
-	FLAC__METADATA_TYPE_APPLICATION = 2
+	FLAC__METADATA_TYPE_APPLICATION = 2,
+	FLAC__METADATA_TYPE_SEEKTABLE = 3
 } FLAC__MetaDataType;
 extern const char *FLAC__MetaDataTypeString[];
 
@@ -367,6 +372,7 @@ extern const unsigned FLAC__STREAM_METADATA_STREAMINFO_LENGTH; /* = 34 bytes */
  */
 typedef struct {
 	int dummy; /* conceptually this is an empty struct since we don't store the padding bytes */
+	           /* empty structs are allowed by C++ but not C, hence the 'dummy' */
 } FLAC__StreamMetaData_Padding;
 
 /*****************************************************************************
@@ -383,6 +389,45 @@ typedef struct {
 
 extern const unsigned FLAC__STREAM_METADATA_APPLICATION_ID_LEN; /* = 32 bits */
 
+/*****************************************************************************
+ *
+ *  64: sample number of target frame
+ *  64: offset, in bytes, of target frame with respect to beginning of first frame
+ *  16: number of samples in the target frame
+ *----- -----------------
+ *  18  bytes total
+ */
+typedef struct {
+	uint64 sample_number;
+	uint64 stream_offset;
+	unsigned frame_samples;
+} FLAC__StreamMetaData_SeekPoint;
+
+extern const unsigned FLAC__STREAM_METADATA_SEEKPOINT_SAMPLE_NUMBER_LEN; /* = 64 bits */
+extern const unsigned FLAC__STREAM_METADATA_SEEKPOINT_STREAM_OFFSET_LEN; /* = 64 bits */
+extern const unsigned FLAC__STREAM_METADATA_SEEKPOINT_FRAME_SAMPLES_LEN; /* = 16 bits */
+extern const unsigned FLAC__STREAM_METADATA_SEEKPOINT_LEN; /* = 18 bytes */
+
+extern const uint64 FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER; /* = 0xffffffffffffffff */
+
+/*****************************************************************************
+ *
+ *      0: num_points is implied by the metadata block 'length' field (i.e. num_points = length / 18)
+ * n*18*8: seek points (n = num_points, 18 is the size of a seek point in bytes)
+ * ------- -----------------
+ *   n*18  bytes total
+ *
+ * NOTE: the seek points must be sorted by ascending sample number.
+ * NOTE: each seek point's sample number must be the first sample of the target frame.
+ * NOTE: each seek point's sample number must be unique within the table.
+ * NOTE: existence of a SEEKTABLE block implies a correct setting of total_samples in the stream_info block.
+ * NOTE: behavior is undefined when more than one SEEKTABLE block is present in a stream.
+ */
+typedef struct {
+	unsigned num_points;
+	FLAC__StreamMetaData_SeekPoint *points;
+} FLAC__StreamMetaData_SeekTable;
+
 /*****************************************************************************
  *
  *  1: =1 if this is the last meta-data block, else =0
@@ -399,6 +444,7 @@ typedef struct {
 		FLAC__StreamMetaData_StreamInfo stream_info;
 		FLAC__StreamMetaData_Padding padding;
 		FLAC__StreamMetaData_Application application;
+		FLAC__StreamMetaData_SeekTable seek_table;
 	} data;
 } FLAC__StreamMetaData;
 
