@@ -17,15 +17,26 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include <stdlib.h> /* for malloc() */
 #include <string.h> /* for memcpy(), memset() */
 #include "private/bitbuffer.h"
 #include "private/bitmath.h"
 #include "private/crc.h"
+#include "FLAC/assert.h"
 
+/*
+ * Along the way you will see two versions of some functions, selected
+ * by a FLAC__NO_MANUAL_INLINING macro.  One is the simplified, more
+ * readable, and slow version, and the other is the same function
+ * where crucial parts have been manually inlined and are much faster.
+ *
+ */
+
+/* This should be at least twice as large as the largest number of bytes required to represent any 'number' (in any encoding) you are going to read. */
 static const unsigned FLAC__BITBUFFER_DEFAULT_CAPACITY = 65536; /* bytes */
 
+#define BYTE_BIT_TO_MASK(b) (((byte)'\x80') >> (b))
+
 #ifdef min
 #undef min
 #endif
@@ -39,8 +50,8 @@ static bool bitbuffer_resize_(FLAC__BitBuffer *bb, unsigned new_capacity)
 {
 	byte *new_buffer;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
 	if(bb->capacity == new_capacity)
 		return true;
@@ -69,7 +80,7 @@ static bool bitbuffer_grow_(FLAC__BitBuffer *bb, unsigned min_bytes_to_add)
 {
 	unsigned new_capacity;
 
-	assert(min_bytes_to_add > 0);
+	FLAC__ASSERT(min_bytes_to_add > 0);
 
 	new_capacity = max(bb->capacity * 4, bb->capacity + min_bytes_to_add);
 	return bitbuffer_resize_(bb, new_capacity);
@@ -77,8 +88,8 @@ static bool bitbuffer_grow_(FLAC__BitBuffer *bb, unsigned min_bytes_to_add)
 
 static bool bitbuffer_ensure_size_(FLAC__BitBuffer *bb, unsigned bits_to_add)
 {
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 	if((bb->capacity<<3) < bb->total_bits + bits_to_add)
 		return bitbuffer_grow_(bb, (bits_to_add>>3)+2);
 	else
@@ -106,7 +117,7 @@ static bool bitbuffer_read_from_client_(FLAC__BitBuffer *bb, bool (*read_callbac
 		if(!bitbuffer_resize_(bb, 16))
 			return false;
 	}
-	/* finally, read in some data; if OK, go back to read_bit_, else fail */
+	/* finally, read in some data */
 	bytes = bb->capacity - bb->bytes;
 	if(!read_callback(bb->buffer+bb->bytes, &bytes, client_data))
 		return false;
@@ -117,7 +128,8 @@ static bool bitbuffer_read_from_client_(FLAC__BitBuffer *bb, bool (*read_callbac
 
 void FLAC__bitbuffer_init(FLAC__BitBuffer *bb)
 {
-	assert(bb != 0);
+	FLAC__ASSERT(bb != 0);
+
 	bb->buffer = 0;
 	bb->capacity = 0;
 	bb->bytes = bb->bits = bb->total_bits = 0;
@@ -126,12 +138,13 @@ void FLAC__bitbuffer_init(FLAC__BitBuffer *bb)
 
 bool FLAC__bitbuffer_init_from(FLAC__BitBuffer *bb, const byte buffer[], unsigned bytes)
 {
-	assert(bb != 0);
+	FLAC__ASSERT(bb != 0);
 	FLAC__bitbuffer_init(bb);
+
 	if(bytes == 0)
 		return true;
 	else {
-		assert(buffer != 0);
+		FLAC__ASSERT(buffer != 0);
 		bb->buffer = (byte*)malloc(sizeof(byte)*bytes);
 		if(bb->buffer == 0)
 			return false;
@@ -146,7 +159,7 @@ bool FLAC__bitbuffer_init_from(FLAC__BitBuffer *bb, const byte buffer[], unsigne
 
 void FLAC__bitbuffer_init_read_crc16(FLAC__BitBuffer *bb, uint16 seed)
 {
-	assert(bb != 0);
+	FLAC__ASSERT(bb != 0);
 
 	bb->read_crc16 = seed;
 }
@@ -156,8 +169,8 @@ bool FLAC__bitbuffer_concatenate_aligned(FLAC__BitBuffer *dest, const FLAC__BitB
 	static const byte mask_[9] = { 0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff };
 	unsigned bits_to_add = src->total_bits - src->total_consumed_bits;
 
-	assert(dest != 0);
-	assert(src != 0);
+	FLAC__ASSERT(dest != 0);
+	FLAC__ASSERT(src != 0);
 
 	if(bits_to_add == 0)
 		return true;
@@ -186,7 +199,8 @@ bool FLAC__bitbuffer_concatenate_aligned(FLAC__BitBuffer *dest, const FLAC__BitB
 
 void FLAC__bitbuffer_free(FLAC__BitBuffer *bb)
 {
-	assert(bb != 0);
+	FLAC__ASSERT(bb != 0);
+
 	if(bb->buffer != 0)
 		free(bb->buffer);
 	bb->buffer = 0;
@@ -232,8 +246,8 @@ bool FLAC__bitbuffer_write_zeroes(FLAC__BitBuffer *bb, unsigned bits)
 {
 	unsigned n, k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
 	if(bits == 0)
 		return true;
@@ -256,28 +270,21 @@ bool FLAC__bitbuffer_write_zeroes(FLAC__BitBuffer *bb, unsigned bits)
 
 bool FLAC__bitbuffer_write_raw_uint32(FLAC__BitBuffer *bb, uint32 val, unsigned bits)
 {
-	static const uint32 mask[] = {
-		0,
-		0x00000001, 0x00000003, 0x00000007, 0x0000000F,
-		0x0000001F, 0x0000003F, 0x0000007F, 0x000000FF,
-		0x000001FF, 0x000003FF, 0x000007FF, 0x00000FFF,
-		0x00001FFF, 0x00003FFF, 0x00007FFF, 0x0000FFFF,
-		0x0001FFFF, 0x0003FFFF, 0x0007FFFF, 0x000FFFFF,
-		0x001FFFFF, 0x003FFFFF, 0x007FFFFF, 0x00FFFFFF,
-		0x01FFFFFF, 0x03FFFFFF, 0x07FFFFFF, 0x0FFFFFFF,
-		0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFF
-	};
 	unsigned n, k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 32);
+	FLAC__ASSERT(bits <= 32);
 	if(bits == 0)
 		return true;
-	if(!bitbuffer_ensure_size_(bb, bits))
-		return false;
-	val &= mask[bits];
+	/* inline the size check so we don't incure a function call unnecessarily */
+	if((bb->capacity<<3) < bb->total_bits + bits) {
+		if(!bitbuffer_ensure_size_(bb, bits))
+			return false;
+	}
+	if(bits < 32) /* @@@ gcc seems to require this because the following line causes incorrect results when bits==32; investigate */
+		val &= (~(0xffffffff << bits)); /* zero-out unused bits */
 	bb->total_bits += bits;
 	while(bits > 0) {
 		n = 8 - bb->bits;
@@ -351,10 +358,10 @@ bool FLAC__bitbuffer_write_raw_uint64(FLAC__BitBuffer *bb, uint64 val, unsigned
 	};
 	unsigned n, k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 64);
+	FLAC__ASSERT(bits <= 64);
 	if(bits == 0)
 		return true;
 	if(!bitbuffer_ensure_size_(bb, bits))
@@ -419,11 +426,11 @@ unsigned FLAC__bitbuffer_rice_bits(int val, unsigned parameter)
 {
 	unsigned msbs, uval;
 
-	/* convert signed to unsigned */
+	/* fold signed to unsigned */
 	if(val < 0)
 		/* equivalent to
 		 *     (unsigned)(((--val) << 1) - 1);
-		 * but without the overflow problem at -MAXINT
+		 * but without the overflow problem at MININT
 		 */
 		uval = (unsigned)(((-(++val)) << 1) + 1);
 	else
@@ -439,13 +446,13 @@ unsigned FLAC__bitbuffer_golomb_bits_signed(int val, unsigned parameter)
 	unsigned bits, msbs, uval;
 	unsigned k;
 
-	assert(parameter > 0);
+	FLAC__ASSERT(parameter > 0);
 
-	/* convert signed to unsigned */
+	/* fold signed to unsigned */
 	if(val < 0)
 		/* equivalent to
 		 *     (unsigned)(((--val) << 1) - 1);
-		 * but without the overflow problem at -MAXINT
+		 * but without the overflow problem at MININT
 		 */
 		uval = (unsigned)(((-(++val)) << 1) + 1);
 	else
@@ -453,7 +460,7 @@ unsigned FLAC__bitbuffer_golomb_bits_signed(int val, unsigned parameter)
 
 	k = FLAC__bitmath_ilog2(parameter);
 	if(parameter == 1u<<k) {
-		assert(k <= 30);
+		FLAC__ASSERT(k <= 30);
 
 		msbs = uval >> k;
 		bits = 1 + k + msbs;
@@ -477,11 +484,11 @@ unsigned FLAC__bitbuffer_golomb_bits_unsigned(unsigned uval, unsigned parameter)
 	unsigned bits, msbs;
 	unsigned k;
 
-	assert(parameter > 0);
+	FLAC__ASSERT(parameter > 0);
 
 	k = FLAC__bitmath_ilog2(parameter);
 	if(parameter == 1u<<k) {
-		assert(k <= 30);
+		FLAC__ASSERT(k <= 30);
 
 		msbs = uval >> k;
 		bits = 1 + k + msbs;
@@ -505,9 +512,9 @@ bool FLAC__bitbuffer_write_symmetric_rice_signed(FLAC__BitBuffer *bb, int val, u
 	unsigned total_bits, interesting_bits, msbs;
 	uint32 pattern;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 31);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 31);
 
 	/* init pattern with the unary end bit and the sign bit */
 	if(val < 0) {
@@ -543,9 +550,9 @@ bool FLAC__bitbuffer_write_symmetric_rice_signed_guarded(FLAC__BitBuffer *bb, in
 	unsigned total_bits, interesting_bits, msbs;
 	uint32 pattern;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 31);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 31);
 
 	*overflow = false;
 
@@ -587,9 +594,9 @@ bool FLAC__bitbuffer_write_symmetric_rice_signed_escape(FLAC__BitBuffer *bb, int
 	unsigned total_bits, val_bits;
 	uint32 pattern;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 31);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 31);
 
 	val_bits = FLAC__bitmath_silog2(val);
 	total_bits = 2 + parameter + 5 + val_bits;
@@ -622,15 +629,15 @@ bool FLAC__bitbuffer_write_rice_signed(FLAC__BitBuffer *bb, int val, unsigned pa
 	unsigned total_bits, interesting_bits, msbs, uval;
 	uint32 pattern;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 30);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 30);
 
-	/* convert signed to unsigned */
+	/* fold signed to unsigned */
 	if(val < 0)
 		/* equivalent to
 		 *     (unsigned)(((--val) << 1) - 1);
-		 * but without the overflow problem at -MAXINT
+		 * but without the overflow problem at MININT
 		 */
 		uval = (unsigned)(((-(++val)) << 1) + 1);
 	else
@@ -662,17 +669,17 @@ bool FLAC__bitbuffer_write_rice_signed_guarded(FLAC__BitBuffer *bb, int val, uns
 	unsigned total_bits, interesting_bits, msbs, uval;
 	uint32 pattern;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 30);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 30);
 
 	*overflow = false;
 
-	/* convert signed to unsigned */
+	/* fold signed to unsigned */
 	if(val < 0)
 		/* equivalent to
 		 *     (unsigned)(((--val) << 1) - 1);
-		 * but without the overflow problem at -MAXINT
+		 * but without the overflow problem at MININT
 		 */
 		uval = (unsigned)(((-(++val)) << 1) + 1);
 	else
@@ -708,15 +715,15 @@ bool FLAC__bitbuffer_write_golomb_signed(FLAC__BitBuffer *bb, int val, unsigned
 	unsigned total_bits, msbs, uval;
 	unsigned k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter > 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter > 0);
 
-	/* convert signed to unsigned */
+	/* fold signed to unsigned */
 	if(val < 0)
 		/* equivalent to
 		 *     (unsigned)(((--val) << 1) - 1);
-		 * but without the overflow problem at -MAXINT
+		 * but without the overflow problem at MININT
 		 */
 		uval = (unsigned)(((-(++val)) << 1) + 1);
 	else
@@ -726,7 +733,7 @@ bool FLAC__bitbuffer_write_golomb_signed(FLAC__BitBuffer *bb, int val, unsigned
 	if(parameter == 1u<<k) {
 		unsigned pattern;
 
-		assert(k <= 30);
+		FLAC__ASSERT(k <= 30);
 
 		msbs = uval >> k;
 		total_bits = 1 + k + msbs;
@@ -776,15 +783,15 @@ bool FLAC__bitbuffer_write_golomb_unsigned(FLAC__BitBuffer *bb, unsigned uval, u
 	unsigned total_bits, msbs;
 	unsigned k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter > 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter > 0);
 
 	k = FLAC__bitmath_ilog2(parameter);
 	if(parameter == 1u<<k) {
 		unsigned pattern;
 
-		assert(k <= 30);
+		FLAC__ASSERT(k <= 30);
 
 		msbs = uval >> k;
 		total_bits = 1 + k + msbs;
@@ -833,10 +840,10 @@ bool FLAC__bitbuffer_write_utf8_uint32(FLAC__BitBuffer *bb, uint32 val)
 {
 	bool ok = 1;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(!(val & 0x80000000)); /* this version only handles 31 bits */
+	FLAC__ASSERT(!(val & 0x80000000)); /* this version only handles 31 bits */
 
 	if(val < 0x80) {
 		return FLAC__bitbuffer_write_raw_uint32(bb, val, 8);
@@ -879,10 +886,10 @@ bool FLAC__bitbuffer_write_utf8_uint64(FLAC__BitBuffer *bb, uint64 val)
 {
 	bool ok = 1;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(!(val & 0xFFFFFFF000000000)); /* this version only handles 36 bits */
+	FLAC__ASSERT(!(val & 0xFFFFFFF000000000)); /* this version only handles 36 bits */
 
 	if(val < 0x80) {
 		return FLAC__bitbuffer_write_raw_uint32(bb, (uint32)val, 8);
@@ -941,17 +948,15 @@ bool FLAC__bitbuffer_zero_pad_to_byte_boundary(FLAC__BitBuffer *bb)
 
 bool FLAC__bitbuffer_peek_bit(FLAC__BitBuffer *bb, unsigned *val, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
-	static const byte mask[] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-
 	/* to avoid a drastic speed penalty we don't:
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(bb->bits == 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(bb->bits == 0);
 	*/
 
 	while(1) {
 		if(bb->total_consumed_bits < bb->total_bits) {
-			*val = (bb->buffer[bb->consumed_bytes] & mask[bb->consumed_bits])? 1 : 0;
+			*val = (bb->buffer[bb->consumed_bytes] & BYTE_BIT_TO_MASK(bb->consumed_bits))? 1 : 0;
 			return true;
 		}
 		else {
@@ -963,17 +968,15 @@ bool FLAC__bitbuffer_peek_bit(FLAC__BitBuffer *bb, unsigned *val, bool (*read_ca
 
 bool FLAC__bitbuffer_read_bit(FLAC__BitBuffer *bb, unsigned *val, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
-	static const byte mask[] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-
 	/* to avoid a drastic speed penalty we don't:
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(bb->bits == 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(bb->bits == 0);
 	*/
 
 	while(1) {
 		if(bb->total_consumed_bits < bb->total_bits) {
-			*val = (bb->buffer[bb->consumed_bytes] & mask[bb->consumed_bits])? 1 : 0;
+			*val = (bb->buffer[bb->consumed_bytes] & BYTE_BIT_TO_MASK(bb->consumed_bits))? 1 : 0;
 			bb->consumed_bits++;
 			if(bb->consumed_bits == 8) {
 				FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
@@ -992,18 +995,16 @@ bool FLAC__bitbuffer_read_bit(FLAC__BitBuffer *bb, unsigned *val, bool (*read_ca
 
 bool FLAC__bitbuffer_read_bit_to_uint32(FLAC__BitBuffer *bb, uint32 *val, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
-	static const byte mask[] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-
 	/* to avoid a drastic speed penalty we don't:
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(bb->bits == 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(bb->bits == 0);
 	*/
 
 	while(1) {
 		if(bb->total_consumed_bits < bb->total_bits) {
 			*val <<= 1;
-			*val |= (bb->buffer[bb->consumed_bytes] & mask[bb->consumed_bits])? 1 : 0;
+			*val |= (bb->buffer[bb->consumed_bytes] & BYTE_BIT_TO_MASK(bb->consumed_bits))? 1 : 0;
 			bb->consumed_bits++;
 			if(bb->consumed_bits == 8) {
 				FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
@@ -1022,18 +1023,16 @@ bool FLAC__bitbuffer_read_bit_to_uint32(FLAC__BitBuffer *bb, uint32 *val, bool (
 
 bool FLAC__bitbuffer_read_bit_to_uint64(FLAC__BitBuffer *bb, uint64 *val, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
-	static const byte mask[] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-
 	/* to avoid a drastic speed penalty we don't:
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(bb->bits == 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(bb->bits == 0);
 	*/
 
 	while(1) {
 		if(bb->total_consumed_bits < bb->total_bits) {
 			*val <<= 1;
-			*val |= (bb->buffer[bb->consumed_bytes] & mask[bb->consumed_bits])? 1 : 0;
+			*val |= (bb->buffer[bb->consumed_bytes] & BYTE_BIT_TO_MASK(bb->consumed_bits))? 1 : 0;
 			bb->consumed_bits++;
 			if(bb->consumed_bits == 8) {
 				FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
@@ -1050,14 +1049,15 @@ bool FLAC__bitbuffer_read_bit_to_uint64(FLAC__BitBuffer *bb, uint64 *val, bool (
 	}
 }
 
-bool FLAC__bitbuffer_read_raw_uint32(FLAC__BitBuffer *bb, uint32 *val, unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+bool FLAC__bitbuffer_read_raw_uint32(FLAC__BitBuffer *bb, uint32 *val, const unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#ifdef FLAC__NO_MANUAL_INLINING
 {
 	unsigned i;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 32);
+	FLAC__ASSERT(bits <= 32);
 
 	*val = 0;
 	for(i = 0; i < bits; i++) {
@@ -1066,43 +1066,164 @@ bool FLAC__bitbuffer_read_raw_uint32(FLAC__BitBuffer *bb, uint32 *val, unsigned
 	}
 	return true;
 }
-
-bool FLAC__bitbuffer_read_raw_int32(FLAC__BitBuffer *bb, int32 *val, unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#else
 {
-	unsigned i;
-	uint32 x;
+	unsigned i, bits_ = bits;
+	uint32 v = 0;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 32);
+	FLAC__ASSERT(bits <= 32);
+	FLAC__ASSERT((bb->capacity*8) * 2 >= bits);
 
-	x = 0;
-	for(i = 0; i < bits; i++) {
-		if(!FLAC__bitbuffer_read_bit_to_uint32(bb, &x, read_callback, client_data))
+	while(bb->total_consumed_bits + bits > bb->total_bits) {
+		if(!bitbuffer_read_from_client_(bb, read_callback, client_data))
 			return false;
 	}
+	if(bb->consumed_bits) {
+		i = 8 - bb->consumed_bits;
+		if(i <= bits_) {
+			v = bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits);
+			bits_ -= i;
+			FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+			bb->consumed_bytes++;
+			bb->consumed_bits = 0;
+			/* we hold off updating bb->total_consumed_bits until the end */
+		}
+		else {
+			*val = (bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits)) >> (i-bits_);
+			bb->consumed_bits += bits_;
+			bb->total_consumed_bits += bits_;
+			return true;
+		}
+	}
+	while(bits_ >= 8) {
+		v <<= 8;
+		v |= bb->buffer[bb->consumed_bytes];
+		bits_ -= 8;
+		FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+		bb->consumed_bytes++;
+		/* bb->consumed_bits is already 0 */
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	if(bits_ > 0) {
+		v <<= bits_;
+		v |= (bb->buffer[bb->consumed_bytes] >> (8-bits_));
+		bb->consumed_bits = bits_;
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	bb->total_consumed_bits += bits;
+	*val = v;
+	return true;
+}
+#endif
+
+bool FLAC__bitbuffer_read_raw_int32(FLAC__BitBuffer *bb, int32 *val, const unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#ifdef FLAC__NO_MANUAL_INLINING
+{
+	unsigned i;
+	uint32 v;
+
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+
+	FLAC__ASSERT(bits <= 32);
+
+	v = 0;
+	for(i = 0; i < bits; i++) {
+		if(!FLAC__bitbuffer_read_bit_to_uint32(bb, &v, read_callback, client_data))
+			return false;
+	}
+
 	/* fix the sign */
 	i = 32 - bits;
 	if(i) {
-		x <<= i;
-		*val = (int32)x;
+		v <<= i;
+		*val = (int32)v;
 		*val >>= i;
 	}
 	else
-		*val = (int32)x;
+		*val = (int32)v;
 
 	return true;
 }
+#else
+{
+	unsigned i, bits_ = bits;
+	uint32 v = 0;
 
-bool FLAC__bitbuffer_read_raw_uint64(FLAC__BitBuffer *bb, uint64 *val, unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+
+	FLAC__ASSERT(bits <= 32);
+	FLAC__ASSERT((bb->capacity*8) * 2 >= bits);
+
+	while(bb->total_consumed_bits + bits > bb->total_bits) {
+		if(!bitbuffer_read_from_client_(bb, read_callback, client_data))
+			return false;
+	}
+	if(bb->consumed_bits) {
+		i = 8 - bb->consumed_bits;
+		if(i <= bits_) {
+			v = bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits);
+			bits_ -= i;
+			FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+			bb->consumed_bytes++;
+			bb->consumed_bits = 0;
+			/* we hold off updating bb->total_consumed_bits until the end */
+		}
+		else {
+			/* bits_ must be < 7 if we get to here */
+			v = (bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits));
+			v <<= (32-i);
+			*val = (int32)v;
+			*val >>= (32-bits_);
+			bb->consumed_bits += bits_;
+			bb->total_consumed_bits += bits_;
+			return true;
+		}
+	}
+	while(bits_ >= 8) {
+		v <<= 8;
+		v |= bb->buffer[bb->consumed_bytes];
+		bits_ -= 8;
+		FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+		bb->consumed_bytes++;
+		/* bb->consumed_bits is already 0 */
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	if(bits_ > 0) {
+		v <<= bits_;
+		v |= (bb->buffer[bb->consumed_bytes] >> (8-bits_));
+		bb->consumed_bits = bits_;
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	bb->total_consumed_bits += bits;
+
+	/* fix the sign */
+	i = 32 - bits;
+	if(i) {
+		v <<= i;
+		*val = (int32)v;
+		*val >>= i;
+	}
+	else
+		*val = (int32)v;
+
+	return true;
+}
+#endif
+
+bool FLAC__bitbuffer_read_raw_uint64(FLAC__BitBuffer *bb, uint64 *val, const unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#ifdef FLAC__NO_MANUAL_INLINING
 {
 	unsigned i;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 64);
+	FLAC__ASSERT(bits <= 64);
 
 	*val = 0;
 	for(i = 0; i < bits; i++) {
@@ -1111,41 +1232,161 @@ bool FLAC__bitbuffer_read_raw_uint64(FLAC__BitBuffer *bb, uint64 *val, unsigned
 	}
 	return true;
 }
+#else
+{
+	unsigned i, bits_ = bits;
+	uint64 v = 0;
 
-bool FLAC__bitbuffer_read_raw_int64(FLAC__BitBuffer *bb, int64 *val, unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+
+	FLAC__ASSERT(bits <= 64);
+	FLAC__ASSERT((bb->capacity*8) * 2 >= bits);
+
+	while(bb->total_consumed_bits + bits > bb->total_bits) {
+		if(!bitbuffer_read_from_client_(bb, read_callback, client_data))
+			return false;
+	}
+	if(bb->consumed_bits) {
+		i = 8 - bb->consumed_bits;
+		if(i <= bits_) {
+			v = bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits);
+			bits_ -= i;
+			FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+			bb->consumed_bytes++;
+			bb->consumed_bits = 0;
+			/* we hold off updating bb->total_consumed_bits until the end */
+		}
+		else {
+			*val = (bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits)) >> (i-bits_);
+			bb->consumed_bits += bits_;
+			bb->total_consumed_bits += bits_;
+			return true;
+		}
+	}
+	while(bits_ >= 8) {
+		v <<= 8;
+		v |= bb->buffer[bb->consumed_bytes];
+		bits_ -= 8;
+		FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+		bb->consumed_bytes++;
+		/* bb->consumed_bits is already 0 */
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	if(bits_ > 0) {
+		v <<= bits_;
+		v |= (bb->buffer[bb->consumed_bytes] >> (8-bits_));
+		bb->consumed_bits = bits_;
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	bb->total_consumed_bits += bits;
+	*val = v;
+	return true;
+}
+#endif
+
+bool FLAC__bitbuffer_read_raw_int64(FLAC__BitBuffer *bb, int64 *val, const unsigned bits, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#ifdef FLAC__NO_MANUAL_INLINING
 {
 	unsigned i;
-	uint64 x;
+	uint64 v;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
-	assert(bits <= 64);
+	FLAC__ASSERT(bits <= 64);
 
-	x = 0;
+	v = 0;
 	for(i = 0; i < bits; i++) {
-		if(!FLAC__bitbuffer_read_bit_to_uint64(bb, &x, read_callback, client_data))
+		if(!FLAC__bitbuffer_read_bit_to_uint64(bb, &v, read_callback, client_data))
 			return false;
 	}
 	/* fix the sign */
 	i = 64 - bits;
 	if(i) {
-		x <<= i;
-		*val = (int64)x;
+		v <<= i;
+		*val = (int64)v;
 		*val >>= i;
 	}
 	else
-		*val = (int64)x;
+		*val = (int64)v;
 
 	return true;
 }
+#else
+{
+	unsigned i, bits_ = bits;
+	uint64 v = 0;
+
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+
+	FLAC__ASSERT(bits <= 64);
+	FLAC__ASSERT((bb->capacity*8) * 2 >= bits);
+
+	while(bb->total_consumed_bits + bits > bb->total_bits) {
+		if(!bitbuffer_read_from_client_(bb, read_callback, client_data))
+			return false;
+	}
+	if(bb->consumed_bits) {
+		i = 8 - bb->consumed_bits;
+		if(i <= bits_) {
+			v = bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits);
+			bits_ -= i;
+			FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+			bb->consumed_bytes++;
+			bb->consumed_bits = 0;
+			/* we hold off updating bb->total_consumed_bits until the end */
+		}
+		else {
+			/* bits_ must be < 7 if we get to here */
+			v = (bb->buffer[bb->consumed_bytes] & (0xff >> bb->consumed_bits));
+			v <<= (64-i);
+			*val = (int64)v;
+			*val >>= (64-bits_);
+			bb->consumed_bits += bits_;
+			bb->total_consumed_bits += bits_;
+			return true;
+		}
+	}
+	while(bits_ >= 8) {
+		v <<= 8;
+		v |= bb->buffer[bb->consumed_bytes];
+		bits_ -= 8;
+		FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+		bb->consumed_bytes++;
+		/* bb->consumed_bits is already 0 */
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	if(bits_ > 0) {
+		v <<= bits_;
+		v |= (bb->buffer[bb->consumed_bytes] >> (8-bits_));
+		bb->consumed_bits = bits_;
+		/* we hold off updating bb->total_consumed_bits until the end */
+	}
+	bb->total_consumed_bits += bits;
+
+	/* fix the sign */
+	i = 64 - bits;
+	if(i) {
+		v <<= i;
+		*val = (int64)v;
+		*val >>= i;
+	}
+	else
+		*val = (int64)v;
+
+	return true;
+}
+#endif
 
 bool FLAC__bitbuffer_read_unary_unsigned(FLAC__BitBuffer *bb, unsigned *val, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
+#ifdef FLAC__NO_MANUAL_INLINING
 {
 	unsigned bit, val_ = 0;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
 	while(1) {
 		if(!FLAC__bitbuffer_read_bit(bb, &bit, read_callback, client_data))
@@ -1158,28 +1399,89 @@ bool FLAC__bitbuffer_read_unary_unsigned(FLAC__BitBuffer *bb, unsigned *val, boo
 	*val = val_;
 	return true;
 }
+#else
+{
+	unsigned i, val_ = 0;
+	unsigned total_bytes_ = (bb->total_bits + 7) / 8;
+	byte b;
+
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+
+	if(bb->consumed_bits) {
+		b = bb->buffer[bb->consumed_bytes] << bb->consumed_bits;
+		if(b) {
+			for(i = 0; !(b & 0x80); i++)
+				b <<= 1;
+			*val = i;
+			i++;
+			bb->consumed_bits += i;
+			bb->total_consumed_bits += i;
+			if(bb->consumed_bits == 8) {
+				FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+				bb->consumed_bytes++;
+				bb->consumed_bits = 0;
+			}
+			return true;
+		}
+		else {
+			val_ = 8 - bb->consumed_bits;
+			FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+			bb->consumed_bytes++;
+			bb->consumed_bits = 0;
+			bb->total_consumed_bits += val_;
+		}
+	}
+	while(1) {
+		if(bb->consumed_bytes >= total_bytes_) {
+			if(!bitbuffer_read_from_client_(bb, read_callback, client_data))
+				return false;
+			total_bytes_ = (bb->total_bits + 7) / 8;
+		}
+		b = bb->buffer[bb->consumed_bytes];
+		if(b) {
+			for(i = 0; !(b & 0x80); i++)
+				b <<= 1;
+			val_ += i;
+			i++;
+			bb->consumed_bits = i;
+			*val = val_;
+			if(i == 8) {
+				FLAC__CRC16_UPDATE(bb->buffer[bb->consumed_bytes], bb->read_crc16);
+				bb->consumed_bytes++;
+				bb->consumed_bits = 0;
+			}
+			bb->total_consumed_bits += i;
+			return true;
+		}
+		else {
+			val_ += 8;
+			FLAC__CRC16_UPDATE(0, bb->read_crc16);
+			bb->consumed_bytes++;
+			/* bb->consumed_bits is already 0 */
+			/* we hold off updating bb->total_consumed_bits until the end */
+			bb->total_consumed_bits += 8;
+		}
+	}
+}
+#endif
 
 bool FLAC__bitbuffer_read_symmetric_rice_signed(FLAC__BitBuffer *bb, int *val, unsigned parameter, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
 	uint32 sign = 0, lsbs = 0, msbs = 0;
-	unsigned bit;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 31);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 31);
 
 	/* read the unary MSBs and end bit */
-	while(1) {
-		if(!FLAC__bitbuffer_read_bit(bb, &bit, read_callback, client_data))
-			return false;
-		if(bit)
-			break;
-		else
-			msbs++;
-	}
+	if(!FLAC__bitbuffer_read_unary_unsigned(bb, &msbs, read_callback, client_data))
+		return false;
+
 	/* read the sign bit */
 	if(!FLAC__bitbuffer_read_bit_to_uint32(bb, &sign, read_callback, client_data))
 		return false;
+
 	/* read the binary LSBs */
 	if(!FLAC__bitbuffer_read_raw_uint32(bb, &lsbs, parameter, read_callback, client_data))
 		return false;
@@ -1195,24 +1497,20 @@ bool FLAC__bitbuffer_read_symmetric_rice_signed(FLAC__BitBuffer *bb, int *val, u
 bool FLAC__bitbuffer_read_rice_signed(FLAC__BitBuffer *bb, int *val, unsigned parameter, bool (*read_callback)(byte buffer[], unsigned *bytes, void *client_data), void *client_data)
 {
 	uint32 lsbs = 0, msbs = 0;
-	unsigned bit, uval;
+	unsigned uval;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
-	assert(parameter <= 31);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
+	FLAC__ASSERT(parameter <= 31);
 
 	/* read the unary MSBs and end bit */
-	while(1) {
-		if(!FLAC__bitbuffer_read_bit(bb, &bit, read_callback, client_data))
-			return false;
-		if(bit)
-			break;
-		else
-			msbs++;
-	}
+	if(!FLAC__bitbuffer_read_unary_unsigned(bb, &msbs, read_callback, client_data))
+		return false;
+
 	/* read the binary LSBs */
 	if(!FLAC__bitbuffer_read_raw_uint32(bb, &lsbs, parameter, read_callback, client_data))
 		return false;
+
 	/* compose the value */
 	uval = (msbs << parameter) | lsbs;
 	if(uval & 1)
@@ -1228,20 +1526,15 @@ bool FLAC__bitbuffer_read_golomb_signed(FLAC__BitBuffer *bb, int *val, unsigned
 	uint32 lsbs = 0, msbs = 0;
 	unsigned bit, uval, k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
 	k = FLAC__bitmath_ilog2(parameter);
 
 	/* read the unary MSBs and end bit */
-	while(1) {
-		if(!FLAC__bitbuffer_read_bit(bb, &bit, read_callback, client_data))
-			return false;
-		if(bit)
-			break;
-		else
-			msbs++;
-	}
+	if(!FLAC__bitbuffer_read_unary_unsigned(bb, &msbs, read_callback, client_data))
+		return false;
+
 	/* read the binary LSBs */
 	if(!FLAC__bitbuffer_read_raw_uint32(bb, &lsbs, k, read_callback, client_data))
 		return false;
@@ -1277,20 +1570,15 @@ bool FLAC__bitbuffer_read_golomb_unsigned(FLAC__BitBuffer *bb, unsigned *val, un
 	uint32 lsbs, msbs = 0;
 	unsigned bit, k;
 
-	assert(bb != 0);
-	assert(bb->buffer != 0);
+	FLAC__ASSERT(bb != 0);
+	FLAC__ASSERT(bb->buffer != 0);
 
 	k = FLAC__bitmath_ilog2(parameter);
 
 	/* read the unary MSBs and end bit */
-	while(1) {
-		if(!FLAC__bitbuffer_read_bit(bb, &bit, read_callback, client_data))
-			return false;
-		if(bit)
-			break;
-		else
-			msbs++;
-	}
+	if(!FLAC__bitbuffer_read_unary_unsigned(bb, &msbs, read_callback, client_data))
+		return false;
+
 	/* read the binary LSBs */
 	if(!FLAC__bitbuffer_read_raw_uint32(bb, &lsbs, k, read_callback, client_data))
 		return false;
