@@ -17,13 +17,13 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include "private/bitmath.h"
+#include "FLAC/assert.h"
 
 unsigned FLAC__bitmath_ilog2(unsigned v)
 {
 	unsigned l = 0;
-	assert(v > 0);
+	FLAC__ASSERT(v > 0);
 	while(v >>= 1)
 		l++;
 	return l;
