@@ -17,11 +17,11 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h> /* for malloc() */
 #include <string.h> /* for strcmp() */
 #include <sys/stat.h> /* for stat() */
+#include "FLAC/assert.h"
 #include "FLAC/file_decoder.h"
 #include "protected/stream_decoder.h"
 #include "private/md5.h"
@@ -39,6 +39,7 @@ typedef struct FLAC__FileDecoderPrivate {
 	byte computed_md5sum[16]; /* this is the sum we computed from the decoded data */
 	/* the rest of these are only used for seeking: */
 	FLAC__StreamMetaData_StreamInfo stream_info; /* we keep this around so we can figure out how to seek quickly */
+	const FLAC__StreamMetaData_SeekTable *seek_table; /* we hold a pointer to the stream decoder's seek table for the same reason */
 	FLAC__Frame last_frame; /* holds the info of the last frame we seeked to */
 	uint64 target_sample;
 } FLAC__FileDecoderPrivate;
@@ -84,13 +85,13 @@ FLAC__FileDecoderState FLAC__file_decoder_init(
 	void *client_data
 )
 {
-	assert(sizeof(int) >= 4); /* we want to die right away if this is not true */
-	assert(decoder != 0);
-	assert(write_callback != 0);
-	assert(metadata_callback != 0);
-	assert(error_callback != 0);
-	assert(decoder->state == FLAC__FILE_DECODER_UNINITIALIZED);
-	assert(decoder->guts == 0);
+	FLAC__ASSERT(sizeof(int) >= 4); /* we want to die right away if this is not true */
+	FLAC__ASSERT(decoder != 0);
+	FLAC__ASSERT(write_callback != 0);
+	FLAC__ASSERT(metadata_callback != 0);
+	FLAC__ASSERT(error_callback != 0);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_UNINITIALIZED);
+	FLAC__ASSERT(decoder->guts == 0);
 
 	decoder->state = FLAC__FILE_DECODER_OK;
 
@@ -105,6 +106,7 @@ FLAC__FileDecoderState FLAC__file_decoder_init(
 	decoder->guts->stream = 0;
 	decoder->guts->file = 0;
 	decoder->guts->filename = 0;
+	decoder->guts->seek_table = 0;
 
 	if(0 == strcmp(filename, "-")) {
 		decoder->guts->file = stdin;
@@ -138,7 +140,7 @@ bool FLAC__file_decoder_finish(FLAC__FileDecoder *decoder)
 {
 	bool md5_failed = false;
 
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 	if(decoder->state == FLAC__FILE_DECODER_UNINITIALIZED)
 		return true;
 	if(decoder->guts != 0) {
@@ -168,7 +170,7 @@ bool FLAC__file_decoder_finish(FLAC__FileDecoder *decoder)
 bool FLAC__file_decoder_process_whole_file(FLAC__FileDecoder *decoder)
 {
 	bool ret;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->guts->stream->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		decoder->state = FLAC__FILE_DECODER_END_OF_FILE;
@@ -176,7 +178,7 @@ bool FLAC__file_decoder_process_whole_file(FLAC__FileDecoder *decoder)
 	if(decoder->state == FLAC__FILE_DECODER_END_OF_FILE)
 		return true;
 
-	assert(decoder->state == FLAC__FILE_DECODER_OK);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_OK);
 
 	ret = FLAC__stream_decoder_process_whole_stream(decoder->guts->stream);
 	if(!ret)
@@ -188,7 +190,7 @@ bool FLAC__file_decoder_process_whole_file(FLAC__FileDecoder *decoder)
 bool FLAC__file_decoder_process_metadata(FLAC__FileDecoder *decoder)
 {
 	bool ret;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->guts->stream->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		decoder->state = FLAC__FILE_DECODER_END_OF_FILE;
@@ -196,7 +198,7 @@ bool FLAC__file_decoder_process_metadata(FLAC__FileDecoder *decoder)
 	if(decoder->state == FLAC__FILE_DECODER_END_OF_FILE)
 		return true;
 
-	assert(decoder->state == FLAC__FILE_DECODER_OK);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_OK);
 
 	ret = FLAC__stream_decoder_process_metadata(decoder->guts->stream);
 	if(!ret)
@@ -208,7 +210,7 @@ bool FLAC__file_decoder_process_metadata(FLAC__FileDecoder *decoder)
 bool FLAC__file_decoder_process_one_frame(FLAC__FileDecoder *decoder)
 {
 	bool ret;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->guts->stream->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		decoder->state = FLAC__FILE_DECODER_END_OF_FILE;
@@ -216,7 +218,7 @@ bool FLAC__file_decoder_process_one_frame(FLAC__FileDecoder *decoder)
 	if(decoder->state == FLAC__FILE_DECODER_END_OF_FILE)
 		return true;
 
-	assert(decoder->state == FLAC__FILE_DECODER_OK);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_OK);
 
 	ret = FLAC__stream_decoder_process_one_frame(decoder->guts->stream);
 	if(!ret)
@@ -228,7 +230,7 @@ bool FLAC__file_decoder_process_one_frame(FLAC__FileDecoder *decoder)
 bool FLAC__file_decoder_process_remaining_frames(FLAC__FileDecoder *decoder)
 {
 	bool ret;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->guts->stream->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		decoder->state = FLAC__FILE_DECODER_END_OF_FILE;
@@ -236,7 +238,7 @@ bool FLAC__file_decoder_process_remaining_frames(FLAC__FileDecoder *decoder)
 	if(decoder->state == FLAC__FILE_DECODER_END_OF_FILE)
 		return true;
 
-	assert(decoder->state == FLAC__FILE_DECODER_OK);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_OK);
 
 	ret = FLAC__stream_decoder_process_remaining_frames(decoder->guts->stream);
 	if(!ret)
@@ -250,8 +252,8 @@ bool FLAC__file_decoder_seek_absolute(FLAC__FileDecoder *decoder, uint64 sample)
 	long filesize;
 	struct stat filestats;
 
-	assert(decoder != 0);
-	assert(decoder->state == FLAC__FILE_DECODER_OK);
+	FLAC__ASSERT(decoder != 0);
+	FLAC__ASSERT(decoder->state == FLAC__FILE_DECODER_OK);
 
 	if(decoder->guts->filename == 0) { /* means the file is stdin... */
 		decoder->state = FLAC__FILE_DECODER_SEEK_ERROR;
@@ -373,6 +375,10 @@ void metadata_callback_(const FLAC__StreamDecoder *decoder, const FLAC__StreamMe
 		if(0 == memcmp(file_decoder->guts->stored_md5sum, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 16))
 			file_decoder->check_md5 = false;
 	}
+	else if(metadata->type == FLAC__METADATA_TYPE_SEEKTABLE) {
+		file_decoder->guts->seek_table = &metadata->data.seek_table;
+	}
+
 	if(file_decoder->state != FLAC__FILE_DECODER_SEEKING)
 		file_decoder->guts->metadata_callback(file_decoder, metadata, file_decoder->guts->client_data);
 }
@@ -388,7 +394,8 @@ void error_callback_(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErro
 
 bool seek_to_absolute_sample_(FLAC__FileDecoder *decoder, long filesize, uint64 target_sample)
 {
-	long l, r, pos, last_pos = -1;
+	long first_frame_offset, lower_bound, upper_bound, pos = -1, last_pos = -1;
+	int i, lower_seek_point = -1, upper_seek_point = -1;
 	unsigned approx_bytes_per_frame;
 	uint64 last_frame_sample = 0xffffffffffffffff;
 	bool needs_seek;
@@ -396,7 +403,7 @@ bool seek_to_absolute_sample_(FLAC__FileDecoder *decoder, long filesize, uint64
 
 	/* we are just guessing here, but we want to guess high, not low */
 	if(decoder->guts->stream_info.max_framesize > 0) {
-		approx_bytes_per_frame = decoder->guts->stream_info.max_framesize + 64;
+		approx_bytes_per_frame = decoder->guts->stream_info.max_framesize;
 	}
 	else if(!is_variable_blocksize_stream) {
 		/* note there are no () around 'decoder->guts->stream_info.bits_per_sample/8' to keep precision up since it's an integer calulation */
@@ -405,33 +412,96 @@ bool seek_to_absolute_sample_(FLAC__FileDecoder *decoder, long filesize, uint64
 	else
 		approx_bytes_per_frame = 1152 * decoder->guts->stream_info.channels * decoder->guts->stream_info.bits_per_sample/8 + 64;
 
-	/* The file pointer is currently at the first frame plus any read
-	   ahead data, so first we get the file pointer, then subtract
-	   uncomsumed bytes to get the position (l) of the first frame
-	   in the file */
-	if(-1 == (l = ftell(decoder->guts->file))) {
+	/*
+	 * The file pointer is currently at the first frame plus any read
+	 * ahead data, so first we get the file pointer, then subtract
+	 * uncomsumed bytes to get the position of the first frame in the
+	 * file.
+	 */
+	if(-1 == (first_frame_offset = ftell(decoder->guts->file))) {
 		decoder->state = FLAC__FILE_DECODER_SEEK_ERROR;
 		return false;
 	}
-	l -= FLAC__stream_decoder_input_bytes_unconsumed(decoder->guts->stream) + 1;
-	if(l < 0)
-		l = 0;
+	first_frame_offset -= FLAC__stream_decoder_input_bytes_unconsumed(decoder->guts->stream);
+	FLAC__ASSERT(first_frame_offset >= 0);
 
-	/* Now we need to use the metadata and the filelength to search to the frame with the correct sample */
-#ifdef _MSC_VER
-	/* with VC++ you have to spoon feed it the casting */
-	pos = l + (long)((double)(int64)target_sample / (double)(int64)decoder->guts->stream_info.total_samples * (double)(filesize-l+1)) - approx_bytes_per_frame;
-#else
-	pos = l + (long)((double)target_sample / (double)decoder->guts->stream_info.total_samples * (double)(filesize-l+1)) - approx_bytes_per_frame;
-#endif
+	/*
+	 * First, we set an upper and lower bound on where in the
+	 * file we will search.  For now we assume the worst case
+	 * scenario, which is our best guess at the beginning of
+	 * the first and last frames.
+	 */
+	lower_bound = first_frame_offset;
+
+	/* calc the upper_bound, beyond which we never want to seek */
 	if(decoder->guts->stream_info.max_framesize > 0)
-		r = filesize - decoder->guts->stream_info.max_framesize - 2;
+		upper_bound = filesize - (decoder->guts->stream_info.max_framesize + 128 + 2); /* 128 for a possible ID3V1 tag, 2 for indexing differences */
 	else
-		r = filesize - ((decoder->guts->stream_info.channels * decoder->guts->stream_info.bits_per_sample * FLAC__MAX_BLOCK_SIZE) / 8 + 64);
-	if(pos >= r)
-		pos = r-1;
-	if(pos < l)
-		pos = l;
+		upper_bound = filesize - ((decoder->guts->stream_info.channels * decoder->guts->stream_info.bits_per_sample * FLAC__MAX_BLOCK_SIZE) / 8 + 128 + 2);
+
+	/*
+	 * Now we refine the bounds if we have a seektable with
+	 * suitable points.  Note that according to the spec they
+	 * must be ordered by ascending sample number.
+	 */
+	if(0 != decoder->guts->seek_table) {
+		/* find the closest seek point <= target_sample, if it exists */
+		for(i = (int)decoder->guts->seek_table->num_points - 1; i >= 0; i--) {
+			if(decoder->guts->seek_table->points[i].sample_number != FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER && decoder->guts->seek_table->points[i].sample_number <= target_sample)
+				break;
+		}
+		if(i >= 0) { /* i.e. we found a suitable seek point... */
+			lower_bound = first_frame_offset + decoder->guts->seek_table->points[i].stream_offset;
+			lower_seek_point = i;
+		}
+
+		/* find the closest seek point > target_sample, if it exists */
+		for(i = 0; i < (int)decoder->guts->seek_table->num_points; i++) {
+			if(decoder->guts->seek_table->points[i].sample_number != FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER && decoder->guts->seek_table->points[i].sample_number > target_sample)
+				break;
+		}
+		if(i < (int)decoder->guts->seek_table->num_points) { /* i.e. we found a suitable seek point... */
+			upper_bound = first_frame_offset + decoder->guts->seek_table->points[i].stream_offset;
+			upper_seek_point = i;
+		}
+	}
+
+	/*
+	 * Now guess at where within those bounds our target
+	 * sample will be.
+	 */
+	if(lower_seek_point >= 0) {
+		/* first see if our sample is within a few frames of the lower seekpoint */
+		if(decoder->guts->seek_table->points[lower_seek_point].sample_number <= target_sample && target_sample < decoder->guts->seek_table->points[lower_seek_point].sample_number + (decoder->guts->seek_table->points[lower_seek_point].frame_samples * 4)) {
+			pos = lower_bound;
+		}
+		else if(upper_seek_point >= 0) {
+			const uint64 target_offset = target_sample - decoder->guts->seek_table->points[lower_seek_point].sample_number;
+			const uint64 range_samples = decoder->guts->seek_table->points[upper_seek_point].sample_number - decoder->guts->seek_table->points[lower_seek_point].sample_number;
+			const long range_bytes = upper_bound - lower_bound;
+#ifdef _MSC_VER
+			/* with VC++ you have to spoon feed it the casting */
+			pos = lower_bound + (long)((double)(int64)target_offset / (double)(int64)range_samples * (double)(range_bytes-1)) - approx_bytes_per_frame;
+#else
+			pos = lower_bound + (long)((double)target_offset / (double)range_samples * (double)(range_bytes-1)) - approx_bytes_per_frame;
+#endif
+		}
+	}
+	if(pos < 0) {
+		/* We need to use the metadata and the filelength to estimate the position of the frame with the correct sample */
+#ifdef _MSC_VER
+		/* with VC++ you have to spoon feed it the casting */
+		pos = first_frame_offset + (long)((double)(int64)target_sample / (double)(int64)decoder->guts->stream_info.total_samples * (double)(filesize-first_frame_offset-1)) - approx_bytes_per_frame;
+#else
+		pos = first_frame_offset + (long)((double)target_sample / (double)decoder->guts->stream_info.total_samples * (double)(filesize-first_frame_offset-1)) - approx_bytes_per_frame;
+#endif
+	}
+
+	/* clip the position to the bounds, lower bound takes precedence */
+	if(pos >= upper_bound)
+		pos = upper_bound-1;
+	if(pos < lower_bound)
+		pos = lower_bound;
 	needs_seek = true;
 
 	decoder->guts->target_sample = target_sample;
@@ -468,7 +538,7 @@ bool seek_to_absolute_sample_(FLAC__FileDecoder *decoder, long filesize, uint64
 					pos -= approx_bytes_per_frame;
 					needs_seek = true;
 				}
-				else {
+				else { /* target_sample >= this_frame_sample + this frame's blocksize */
 					last_pos = pos;
 					if(-1 == (pos = ftell(decoder->guts->file))) {
 						decoder->state = FLAC__FILE_DECODER_SEEK_ERROR;
@@ -478,8 +548,8 @@ bool seek_to_absolute_sample_(FLAC__FileDecoder *decoder, long filesize, uint64
 					needs_seek = false;
 				}
 			}
-			if(pos < l)
-				pos = l;
+			if(pos < lower_bound)
+				pos = lower_bound;
 			last_frame_sample = this_frame_sample;
 		}
 	}
