@@ -17,9 +17,9 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include <math.h>
 #include "private/fixed.h"
+#include "FLAC/assert.h"
 
 #ifndef M_LN2
 /* math.h in VC++ doesn't seem to have this (how Microsoft is that?) */
@@ -42,21 +42,16 @@ unsigned FLAC__fixed_compute_best_predictor(const int32 data[], unsigned data_le
 	int32 last_error_1 = data[-1] - data[-2];
 	int32 last_error_2 = last_error_1 - (data[-2] - data[-3]);
 	int32 last_error_3 = last_error_2 - (data[-2] - 2*data[-3] + data[-4]);
-	int32 error_0, error_1, error_2, error_3, error_4;
+	int32 error, save;
 	uint32 total_error_0 = 0, total_error_1 = 0, total_error_2 = 0, total_error_3 = 0, total_error_4 = 0;
 	unsigned i, order;
 
 	for(i = 0; i < data_len; i++) {
-		error_0 = data[i]               ; total_error_0 += local_abs(error_0);
-		error_1 = error_0 - last_error_0; total_error_1 += local_abs(error_1);
-		error_2 = error_1 - last_error_1; total_error_2 += local_abs(error_2);
-		error_3 = error_2 - last_error_2; total_error_3 += local_abs(error_3);
-		error_4 = error_3 - last_error_3; total_error_4 += local_abs(error_4);
-
-		last_error_0 = error_0;
-		last_error_1 = error_1;
-		last_error_2 = error_2;
-		last_error_3 = error_3;
+		error  = data[i]     ; total_error_0 += local_abs(error);                      save = error;
+		error -= last_error_0; total_error_1 += local_abs(error); last_error_0 = save; save = error;
+		error -= last_error_1; total_error_2 += local_abs(error); last_error_1 = save; save = error;
+		error -= last_error_2; total_error_3 += local_abs(error); last_error_2 = save; save = error;
+		error -= last_error_3; total_error_4 += local_abs(error); last_error_3 = save;
 	}
 
 	if(total_error_0 < min(min(min(total_error_1, total_error_2), total_error_3), total_error_4))
@@ -70,23 +65,11 @@ unsigned FLAC__fixed_compute_best_predictor(const int32 data[], unsigned data_le
 	else
 		order = 4;
 
-	/* Estimate the expected number of bits per residual signal sample. */
-	/* 'total_error*' is linearly related to the variance of the residual */
-	/* signal, so we use it directly to compute E(|x|) */
-#ifdef _MSC_VER
-	/* with VC++ you have to spoon feed it the casting */
-	residual_bits_per_sample[0] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_0  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[1] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_1  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[2] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_2  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[3] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_3  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[4] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_4  / (real) data_len) / M_LN2 : 0.0);
-#else
-	residual_bits_per_sample[0] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_0  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[1] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_1  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[2] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_2  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[3] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_3  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[4] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_4  / (real) data_len) / M_LN2 : 0.0);
-#endif
+	residual_bits_per_sample[0] = (real)((data_len > 0 && total_error_0 > 0) ? log(M_LN2 * (double)total_error_0 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[1] = (real)((data_len > 0 && total_error_1 > 0) ? log(M_LN2 * (double)total_error_1 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[2] = (real)((data_len > 0 && total_error_2 > 0) ? log(M_LN2 * (double)total_error_2 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[3] = (real)((data_len > 0 && total_error_3 > 0) ? log(M_LN2 * (double)total_error_3 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[4] = (real)((data_len > 0 && total_error_4 > 0) ? log(M_LN2 * (double)total_error_4 / (double) data_len) / M_LN2 : 0.0);
 
 	return order;
 }
@@ -97,7 +80,7 @@ unsigned FLAC__fixed_compute_best_predictor_slow(const int32 data[], unsigned da
 	int32 last_error_1 = data[-1] - data[-2];
 	int32 last_error_2 = last_error_1 - (data[-2] - data[-3]);
 	int32 last_error_3 = last_error_2 - (data[-2] - 2*data[-3] + data[-4]);
-	int32 error_0, error_1, error_2, error_3, error_4;
+	int32 error, save;
 	/* total_error_* are 64-bits to avoid overflow when encoding
 	 * erratic signals when the bits-per-sample and blocksize are
 	 * large.
@@ -106,16 +89,11 @@ unsigned FLAC__fixed_compute_best_predictor_slow(const int32 data[], unsigned da
 	unsigned i, order;
 
 	for(i = 0; i < data_len; i++) {
-		error_0 = data[i]               ; total_error_0 += local_abs(error_0);
-		error_1 = error_0 - last_error_0; total_error_1 += local_abs(error_1);
-		error_2 = error_1 - last_error_1; total_error_2 += local_abs(error_2);
-		error_3 = error_2 - last_error_2; total_error_3 += local_abs(error_3);
-		error_4 = error_3 - last_error_3; total_error_4 += local_abs(error_4);
-
-		last_error_0 = error_0;
-		last_error_1 = error_1;
-		last_error_2 = error_2;
-		last_error_3 = error_3;
+		error  = data[i]     ; total_error_0 += local_abs(error);                      save = error;
+		error -= last_error_0; total_error_1 += local_abs(error); last_error_0 = save; save = error;
+		error -= last_error_1; total_error_2 += local_abs(error); last_error_1 = save; save = error;
+		error -= last_error_2; total_error_3 += local_abs(error); last_error_2 = save; save = error;
+		error -= last_error_3; total_error_4 += local_abs(error); last_error_3 = save;
 	}
 
 	if(total_error_0 < min(min(min(total_error_1, total_error_2), total_error_3), total_error_4))
@@ -134,17 +112,17 @@ unsigned FLAC__fixed_compute_best_predictor_slow(const int32 data[], unsigned da
 	/* signal, so we use it directly to compute E(|x|) */
 #ifdef _MSC_VER
 	/* with VC++ you have to spoon feed it the casting */
-	residual_bits_per_sample[0] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_0  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[1] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_1  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[2] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_2  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[3] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_3  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[4] = (real)((data_len > 0) ? log(M_LN2 * (real)(int64)total_error_4  / (real) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[0] = (real)((data_len > 0 && total_error_0 > 0) ? log(M_LN2 * (double)(int64)total_error_0 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[1] = (real)((data_len > 0 && total_error_1 > 0) ? log(M_LN2 * (double)(int64)total_error_1 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[2] = (real)((data_len > 0 && total_error_2 > 0) ? log(M_LN2 * (double)(int64)total_error_2 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[3] = (real)((data_len > 0 && total_error_3 > 0) ? log(M_LN2 * (double)(int64)total_error_3 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[4] = (real)((data_len > 0 && total_error_4 > 0) ? log(M_LN2 * (double)(int64)total_error_4 / (double) data_len) / M_LN2 : 0.0);
 #else
-	residual_bits_per_sample[0] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_0  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[1] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_1  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[2] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_2  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[3] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_3  / (real) data_len) / M_LN2 : 0.0);
-	residual_bits_per_sample[4] = (real)((data_len > 0) ? log(M_LN2 * (real)total_error_4  / (real) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[0] = (real)((data_len > 0 && total_error_0 > 0) ? log(M_LN2 * (double)total_error_0 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[1] = (real)((data_len > 0 && total_error_1 > 0) ? log(M_LN2 * (double)total_error_1 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[2] = (real)((data_len > 0 && total_error_2 > 0) ? log(M_LN2 * (double)total_error_2 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[3] = (real)((data_len > 0 && total_error_3 > 0) ? log(M_LN2 * (double)total_error_3 / (double) data_len) / M_LN2 : 0.0);
+	residual_bits_per_sample[4] = (real)((data_len > 0 && total_error_4 > 0) ? log(M_LN2 * (double)total_error_4 / (double) data_len) / M_LN2 : 0.0);
 #endif
 
 	return order;
@@ -152,76 +130,76 @@ unsigned FLAC__fixed_compute_best_predictor_slow(const int32 data[], unsigned da
 
 void FLAC__fixed_compute_residual(const int32 data[], unsigned data_len, unsigned order, int32 residual[])
 {
-	unsigned i;
+	int i, idata_len = (int)data_len;
 
 	switch(order) {
 		case 0:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				residual[i] = data[i];
 			}
 			break;
 		case 1:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				residual[i] = data[i] - data[i-1];
 			}
 			break;
 		case 2:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* == data[i] - 2*data[i-1] + data[i-2] */
 				residual[i] = data[i] - (data[i-1] << 1) + data[i-2];
 			}
 			break;
 		case 3:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* == data[i] - 3*data[i-1] + 3*data[i-2] - data[i-3] */
 				residual[i] = data[i] - (((data[i-1]-data[i-2])<<1) + (data[i-1]-data[i-2])) - data[i-3];
 			}
 			break;
 		case 4:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* == data[i] - 4*data[i-1] + 6*data[i-2] - 4*data[i-3] + data[i-4] */
 				residual[i] = data[i] - ((data[i-1]+data[i-3])<<2) + ((data[i-2]<<2) + (data[i-2]<<1)) + data[i-4];
 			}
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 	}
 }
 
 void FLAC__fixed_restore_signal(const int32 residual[], unsigned data_len, unsigned order, int32 data[])
 {
-	unsigned i;
+	int i, idata_len = (int)data_len;
 
 	switch(order) {
 		case 0:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				data[i] = residual[i];
 			}
 			break;
 		case 1:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				data[i] = residual[i] + data[i-1];
 			}
 			break;
 		case 2:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* == residual[i] + 2*data[i-1] - data[i-2] */
 				data[i] = residual[i] + (data[i-1]<<1) - data[i-2];
 			}
 			break;
 		case 3:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* residual[i] + 3*data[i-1] - 3*data[i-2]) + data[i-3] */
 				data[i] = residual[i] + (((data[i-1]-data[i-2])<<1) + (data[i-1]-data[i-2])) + data[i-3];
 			}
 			break;
 		case 4:
-			for(i = 0; i < data_len; i++) {
+			for(i = 0; i < idata_len; i++) {
 				/* == residual[i] + 4*data[i-1] - 6*data[i-2] + 4*data[i-3] - data[i-4] */
 				data[i] = residual[i] + ((data[i-1]+data[i-3])<<2) - ((data[i-2]<<2) + (data[i-2]<<1)) - data[i-4];
 			}
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 	}
 }
