@@ -22,10 +22,12 @@
 
 #include "bitbuffer.h"
 #include "bitmath.h"
+#include "cpu.h"
 #include "crc.h"
 #include "encoder_framing.h"
 #include "fixed.h"
 #include "lpc.h"
 #include "md5.h"
+#include "memory.h"
 
 #endif
