@@ -17,9 +17,9 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include <math.h>
 #include <stdio.h>
+#include "FLAC/assert.h"
 #include "FLAC/format.h"
 #include "private/lpc.h"
 
@@ -28,19 +28,48 @@
 #define M_LN2 0.69314718055994530942
 #endif
 
+#define LOCAL_FABS(x) ((x)<0.0? -(x):(x))
+
 void FLAC__lpc_compute_autocorrelation(const real data[], unsigned data_len, unsigned lag, real autoc[])
 {
+	/* a readable, but slower, version */
+#if 0
 	real d;
 	unsigned i;
 
-	assert(lag > 0);
-	assert(lag <= data_len);
+	FLAC__ASSERT(lag > 0);
+	FLAC__ASSERT(lag <= data_len);
 
 	while(lag--) {
 		for(i = lag, d = 0.0; i < data_len; i++)
 			d += data[i] * data[i - lag];
 		autoc[lag] = d;
 	}
+#endif
+
+	/*
+	 * this version tends to run faster because of better data locality
+	 * ('data_len' is usually much larger than 'lag')
+	 */
+	real d;
+	unsigned sample, coeff;
+	const unsigned limit = data_len - lag;
+
+	FLAC__ASSERT(lag > 0);
+	FLAC__ASSERT(lag <= data_len);
+
+	for(coeff = 0; coeff < lag; coeff++)
+		autoc[coeff] = 0.0;
+	for(sample = 0; sample <= limit; sample++) {
+		d = data[sample];
+		for(coeff = 0; coeff < lag; coeff++)
+			autoc[coeff] += d * data[sample+coeff];
+	}
+	for(; sample < data_len; sample++) {
+		d = data[sample];
+		for(coeff = 0; coeff < data_len - sample; coeff++)
+			autoc[coeff] += d * data[sample+coeff];
+	}
 }
 
 void FLAC__lpc_compute_lp_coefficients(const real autoc[], unsigned max_order, real lp_coeff[][FLAC__MAX_LPC_ORDER], real error[])
@@ -48,15 +77,15 @@ void FLAC__lpc_compute_lp_coefficients(const real autoc[], unsigned max_order, r
 	unsigned i, j;
 	real r, err, ref[FLAC__MAX_LPC_ORDER], lpc[FLAC__MAX_LPC_ORDER];
 
-	assert(0 < max_order);
-	assert(max_order <= FLAC__MAX_LPC_ORDER);
-	assert(autoc[0] != 0.0);
+	FLAC__ASSERT(0 < max_order);
+	FLAC__ASSERT(max_order <= FLAC__MAX_LPC_ORDER);
+	FLAC__ASSERT(autoc[0] != 0.0);
 
 	err = autoc[0];
 
 	for(i = 0; i < max_order; i++) {
 		/* Sum up this iteration's reflection coefficient. */
-		r =- autoc[i+1];
+		r = -autoc[i+1];
 		for(j = 0; j < i; j++)
 			r -= lpc[j] * autoc[i-j];
 		ref[i] = (r/=err);
@@ -75,7 +104,7 @@ void FLAC__lpc_compute_lp_coefficients(const real autoc[], unsigned max_order, r
 
 		/* save this order */
 		for(j = 0; j <= i; j++)
-			lp_coeff[i][j] = -lpc[j]; /* negate to get FIR filter coeffs */
+			lp_coeff[i][j] = -lpc[j]; /* negate FIR filter coeff to get predictor coeff */
 		error[i] = err;
 	}
 }
@@ -83,13 +112,13 @@ void FLAC__lpc_compute_lp_coefficients(const real autoc[], unsigned max_order, r
 int FLAC__lpc_quantize_coefficients(const real lp_coeff[], unsigned order, unsigned precision, unsigned bits_per_sample, int32 qlp_coeff[], int *shift)
 {
 	unsigned i;
-	real d, cmax = -1e99;
+	real d, cmax = -1e32;
 
-	assert(bits_per_sample > 0);
-	assert(bits_per_sample <= sizeof(int32)*8);
-	assert(precision > 0);
-	assert(precision >= FLAC__MIN_QLP_COEFF_PRECISION);
-	assert(precision + bits_per_sample < sizeof(int32)*8);
+	FLAC__ASSERT(bits_per_sample > 0);
+	FLAC__ASSERT(bits_per_sample <= sizeof(int32)*8);
+	FLAC__ASSERT(precision > 0);
+	FLAC__ASSERT(precision >= FLAC__MIN_QLP_COEFF_PRECISION);
+	FLAC__ASSERT(precision + bits_per_sample < sizeof(int32)*8);
 #ifdef NDEBUG
 	(void)bits_per_sample; /* silence compiler warning about unused parameter */
 #endif
@@ -100,7 +129,7 @@ int FLAC__lpc_quantize_coefficients(const real lp_coeff[], unsigned order, unsig
 	for(i = 0; i < order; i++) {
 		if(lp_coeff[i] == 0.0)
 			continue;
-		d = fabs(lp_coeff[i]);
+		d = LOCAL_FABS(lp_coeff[i]);
 		if(d > cmax)
 			cmax = d;
 	}
@@ -129,30 +158,30 @@ int FLAC__lpc_quantize_coefficients(const real lp_coeff[], unsigned order, unsig
 
 void FLAC__lpc_compute_residual_from_qlp_coefficients(const int32 data[], unsigned data_len, const int32 qlp_coeff[], unsigned order, int lp_quantization, int32 residual[])
 {
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 	int64 sumo;
 #endif
 	unsigned i, j;
 	int32 sum;
 	const int32 *history;
 
-#ifdef FLAC_OVERFLOW_DETECT_VERBOSE
+#ifdef FLAC__OVERFLOW_DETECT_VERBOSE
 	fprintf(stderr,"FLAC__lpc_compute_residual_from_qlp_coefficients: data_len=%d, order=%u, lpq=%d",data_len,order,lp_quantization);
 	for(i=0;i<order;i++)
 		fprintf(stderr,", q[%u]=%d",i,qlp_coeff[i]);
 	fprintf(stderr,"\n");
 #endif
-	assert(order > 0);
+	FLAC__ASSERT(order > 0);
 
 	for(i = 0; i < data_len; i++) {
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 		sumo = 0;
 #endif
 		sum = 0;
 		history = data;
 		for(j = 0; j < order; j++) {
 			sum += qlp_coeff[j] * (*(--history));
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 			sumo += (int64)qlp_coeff[j] * (int64)(*history);
 			if(sumo > 2147483647ll || sumo < -2147483648ll) {
 				fprintf(stderr,"FLAC__lpc_compute_residual_from_qlp_coefficients: OVERFLOW, i=%u, j=%u, c=%d, d=%d, sumo=%lld\n",i,j,qlp_coeff[j],*history,sumo);
@@ -174,30 +203,30 @@ void FLAC__lpc_compute_residual_from_qlp_coefficients(const int32 data[], unsign
 
 void FLAC__lpc_restore_signal(const int32 residual[], unsigned data_len, const int32 qlp_coeff[], unsigned order, int lp_quantization, int32 data[])
 {
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 	int64 sumo;
 #endif
 	unsigned i, j;
 	int32 sum;
 	const int32 *history;
 
-#ifdef FLAC_OVERFLOW_DETECT_VERBOSE
+#ifdef FLAC__OVERFLOW_DETECT_VERBOSE
 	fprintf(stderr,"FLAC__lpc_restore_signal: data_len=%d, order=%u, lpq=%d",data_len,order,lp_quantization);
 	for(i=0;i<order;i++)
 		fprintf(stderr,", q[%u]=%d",i,qlp_coeff[i]);
 	fprintf(stderr,"\n");
 #endif
-	assert(order > 0);
+	FLAC__ASSERT(order > 0);
 
 	for(i = 0; i < data_len; i++) {
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 		sumo = 0;
 #endif
 		sum = 0;
 		history = data;
 		for(j = 0; j < order; j++) {
 			sum += qlp_coeff[j] * (*(--history));
-#ifdef FLAC_OVERFLOW_DETECT
+#ifdef FLAC__OVERFLOW_DETECT
 			sumo += (int64)qlp_coeff[j] * (int64)(*history);
 			if(sumo > 2147483647ll || sumo < -2147483648ll) {
 				fprintf(stderr,"FLAC__lpc_restore_signal: OVERFLOW, i=%u, j=%u, c=%d, d=%d, sumo=%lld\n",i,j,qlp_coeff[j],*history,sumo);
@@ -219,20 +248,39 @@ void FLAC__lpc_restore_signal(const int32 residual[], unsigned data_len, const i
 
 real FLAC__lpc_compute_expected_bits_per_residual_sample(real lpc_error, unsigned total_samples)
 {
-	real escale;
+	real error_scale;
 
-	assert(lpc_error >= 0.0); /* the error can never be negative */
-	assert(total_samples > 0);
+	FLAC__ASSERT(total_samples > 0);
 
-	escale = 0.5 * M_LN2 * M_LN2 / (real)total_samples;
+	error_scale = 0.5 * M_LN2 * M_LN2 / (real)total_samples;
 
 	if(lpc_error > 0.0) {
-		real bps = 0.5 * log(escale * lpc_error) / M_LN2;
+		real bps = 0.5 * log(error_scale * lpc_error) / M_LN2;
 		if(bps >= 0.0)
 			return bps;
 		else
 			return 0.0;
 	}
+	else if(lpc_error < 0.0) { /* error should not be negative but can happen due to inadequate float resolution */
+		return 1e32;
+	}
+	else {
+		return 0.0;
+	}
+}
+
+real FLAC__lpc_compute_expected_bits_per_residual_sample_with_error_scale(real lpc_error, real error_scale)
+{
+	if(lpc_error > 0.0) {
+		real bps = 0.5 * log(error_scale * lpc_error) / M_LN2;
+		if(bps >= 0.0)
+			return bps;
+		else
+			return 0.0;
+	}
+	else if(lpc_error < 0.0) { /* error should not be negative but can happen due to inadequate float resolution */
+		return 1e32;
+	}
 	else {
 		return 0.0;
 	}
@@ -241,15 +289,18 @@ real FLAC__lpc_compute_expected_bits_per_residual_sample(real lpc_error, unsigne
 unsigned FLAC__lpc_compute_best_order(const real lpc_error[], unsigned max_order, unsigned total_samples, unsigned bits_per_signal_sample)
 {
 	unsigned order, best_order;
-	real best_bits, tmp_bits;
+	real best_bits, tmp_bits, error_scale;
 
-	assert(max_order > 0);
+	FLAC__ASSERT(max_order > 0);
+	FLAC__ASSERT(total_samples > 0);
+
+	error_scale = 0.5 * M_LN2 * M_LN2 / (real)total_samples;
 
 	best_order = 0;
-	best_bits = FLAC__lpc_compute_expected_bits_per_residual_sample(lpc_error[0], total_samples) * (real)total_samples;
+	best_bits = FLAC__lpc_compute_expected_bits_per_residual_sample_with_error_scale(lpc_error[0], error_scale) * (real)total_samples;
 
 	for(order = 1; order < max_order; order++) {
-		tmp_bits = FLAC__lpc_compute_expected_bits_per_residual_sample(lpc_error[order], total_samples) * (real)(total_samples - order) + (real)(order * bits_per_signal_sample);
+		tmp_bits = FLAC__lpc_compute_expected_bits_per_residual_sample_with_error_scale(lpc_error[order], error_scale) * (real)(total_samples - order) + (real)(order * bits_per_signal_sample);
 		if(tmp_bits < best_bits) {
 			best_order = order;
 			best_bits = tmp_bits;
