@@ -17,12 +17,13 @@
  * Boston, MA  02111-1307, USA.
  */
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h> /* for malloc() */
 #include <string.h> /* for memset/memcpy() */
+#include "FLAC/assert.h"
 #include "FLAC/stream_decoder.h"
 #include "private/bitbuffer.h"
+#include "private/cpu.h"
 #include "private/crc.h"
 #include "private/fixed.h"
 #include "private/lpc.h"
@@ -32,6 +33,8 @@ typedef struct FLAC__StreamDecoderPrivate {
 	FLAC__StreamDecoderWriteStatus (*write_callback)(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const int32 *buffer[], void *client_data);
 	void (*metadata_callback)(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetaData *metadata, void *client_data);
 	void (*error_callback)(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
+	void (*local_lpc_restore_signal)(const int32 residual[], unsigned data_len, const int32 qlp_coeff[], unsigned order, int lp_quantization, int32 data[]);
+	void (*local_lpc_restore_signal_16bit)(const int32 residual[], unsigned data_len, const int32 qlp_coeff[], unsigned order, int lp_quantization, int32 data[]);
 	void *client_data;
 	FLAC__BitBuffer input;
 	int32 *output[FLAC__MAX_CHANNELS];
@@ -39,12 +42,14 @@ typedef struct FLAC__StreamDecoderPrivate {
 	unsigned output_capacity, output_channels;
 	uint32 last_frame_number;
 	uint64 samples_decoded;
-	bool has_stream_header;
-	FLAC__StreamMetaData stream_header;
+	bool has_stream_info, has_seek_table;
+	FLAC__StreamMetaData stream_info;
+	FLAC__StreamMetaData seek_table;
 	FLAC__Frame frame;
+	bool cached; /* true if there is a byte in lookahead */
+	FLAC__CPUInfo cpuinfo;
 	byte header_warmup[2]; /* contains the sync code and reserved bits */
 	byte lookahead; /* temp storage when we need to look ahead one byte in the stream */
-	bool cached; /* true if there is a byte in lookahead */
 } FLAC__StreamDecoderPrivate;
 
 static byte ID3V2_TAG_[3] = { 'I', 'D', '3' };
@@ -120,14 +125,14 @@ FLAC__StreamDecoderState FLAC__stream_decoder_init(
 {
 	unsigned i;
 
-	assert(sizeof(int) >= 4); /* we want to die right away if this is not true */
-	assert(decoder != 0);
-	assert(read_callback != 0);
-	assert(write_callback != 0);
-	assert(metadata_callback != 0);
-	assert(error_callback != 0);
-	assert(decoder->state == FLAC__STREAM_DECODER_UNINITIALIZED);
-	assert(decoder->guts == 0);
+	FLAC__ASSERT(sizeof(int) >= 4); /* we want to die right away if this is not true */
+	FLAC__ASSERT(decoder != 0);
+	FLAC__ASSERT(read_callback != 0);
+	FLAC__ASSERT(write_callback != 0);
+	FLAC__ASSERT(metadata_callback != 0);
+	FLAC__ASSERT(error_callback != 0);
+	FLAC__ASSERT(decoder->state == FLAC__STREAM_DECODER_UNINITIALIZED);
+	FLAC__ASSERT(decoder->guts == 0);
 
 	decoder->state = FLAC__STREAM_DECODER_SEARCH_FOR_METADATA;
 
@@ -152,19 +157,49 @@ FLAC__StreamDecoderState FLAC__stream_decoder_init(
 	decoder->guts->output_channels = 0;
 	decoder->guts->last_frame_number = 0;
 	decoder->guts->samples_decoded = 0;
-	decoder->guts->has_stream_header = false;
+	decoder->guts->has_stream_info = false;
+	decoder->guts->has_seek_table = false;
 	decoder->guts->cached = false;
 
+	/*
+	 * get the CPU info and set the function pointers
+	 */
+	FLAC__cpu_info(&decoder->guts->cpuinfo);
+	/* first default to the non-asm routines */
+	decoder->guts->local_lpc_restore_signal = FLAC__lpc_restore_signal;
+	decoder->guts->local_lpc_restore_signal_16bit = FLAC__lpc_restore_signal;
+	/* now override with asm where appropriate */
+#ifndef FLAC__NO_ASM
+	FLAC__ASSERT(decoder->guts->cpuinfo.use_asm);
+#ifdef FLAC__CPU_IA32
+	FLAC__ASSERT(decoder->guts->cpuinfo.type == FLAC__CPUINFO_TYPE_IA32);
+#ifdef FLAC__HAS_NASM
+	if(decoder->guts->cpuinfo.data.ia32.mmx) {
+		decoder->guts->local_lpc_restore_signal = FLAC__lpc_restore_signal_asm_i386;
+		decoder->guts->local_lpc_restore_signal_16bit = FLAC__lpc_restore_signal_asm_i386_mmx;
+	}
+	else {
+		decoder->guts->local_lpc_restore_signal = FLAC__lpc_restore_signal_asm_i386;
+		decoder->guts->local_lpc_restore_signal_16bit = FLAC__lpc_restore_signal_asm_i386;
+	}
+#endif
+#endif
+#endif
+
 	return decoder->state;
 }
 
 void FLAC__stream_decoder_finish(FLAC__StreamDecoder *decoder)
 {
 	unsigned i;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 	if(decoder->state == FLAC__STREAM_DECODER_UNINITIALIZED)
 		return;
 	if(decoder->guts != 0) {
+		if(decoder->guts->has_seek_table) {
+			free(decoder->guts->seek_table.data.seek_table.points);
+			decoder->guts->seek_table.data.seek_table.points = 0;
+		}
 		FLAC__bitbuffer_free(&decoder->guts->input);
 		for(i = 0; i < FLAC__MAX_CHANNELS; i++) {
 			if(decoder->guts->output[i] != 0) {
@@ -184,7 +219,7 @@ void FLAC__stream_decoder_finish(FLAC__StreamDecoder *decoder)
 
 bool FLAC__stream_decoder_flush(FLAC__StreamDecoder *decoder)
 {
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(!FLAC__bitbuffer_clear(&decoder->guts->input)) {
 		decoder->state = FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
@@ -196,7 +231,7 @@ bool FLAC__stream_decoder_flush(FLAC__StreamDecoder *decoder)
 
 bool FLAC__stream_decoder_reset(FLAC__StreamDecoder *decoder)
 {
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(!FLAC__stream_decoder_flush(decoder)) {
 		decoder->state = FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
@@ -204,18 +239,20 @@ bool FLAC__stream_decoder_reset(FLAC__StreamDecoder *decoder)
 	}
 	decoder->state = FLAC__STREAM_DECODER_SEARCH_FOR_METADATA;
 
+	decoder->guts->samples_decoded = 0;
+
 	return true;
 }
 
 bool FLAC__stream_decoder_process_whole_stream(FLAC__StreamDecoder *decoder)
 {
 	bool dummy;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		return true;
 
-	assert(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA);
+	FLAC__ASSERT(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA);
 
 	if(!FLAC__stream_decoder_reset(decoder)) {
 		decoder->state = FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
@@ -243,19 +280,19 @@ bool FLAC__stream_decoder_process_whole_stream(FLAC__StreamDecoder *decoder)
 			case FLAC__STREAM_DECODER_END_OF_STREAM:
 				return true;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 	}
 }
 
 bool FLAC__stream_decoder_process_metadata(FLAC__StreamDecoder *decoder)
 {
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		return true;
 
-	assert(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA);
+	FLAC__ASSERT(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA);
 
 	if(!FLAC__stream_decoder_reset(decoder)) {
 		decoder->state = FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
@@ -278,7 +315,7 @@ bool FLAC__stream_decoder_process_metadata(FLAC__StreamDecoder *decoder)
 			case FLAC__STREAM_DECODER_END_OF_STREAM:
 				return true;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 	}
 }
@@ -286,12 +323,12 @@ bool FLAC__stream_decoder_process_metadata(FLAC__StreamDecoder *decoder)
 bool FLAC__stream_decoder_process_one_frame(FLAC__StreamDecoder *decoder)
 {
 	bool got_a_frame;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		return true;
 
-	assert(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC);
+	FLAC__ASSERT(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC);
 
 	while(1) {
 		switch(decoder->state) {
@@ -308,7 +345,7 @@ bool FLAC__stream_decoder_process_one_frame(FLAC__StreamDecoder *decoder)
 			case FLAC__STREAM_DECODER_END_OF_STREAM:
 				return true;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 	}
 }
@@ -316,12 +353,12 @@ bool FLAC__stream_decoder_process_one_frame(FLAC__StreamDecoder *decoder)
 bool FLAC__stream_decoder_process_remaining_frames(FLAC__StreamDecoder *decoder)
 {
 	bool dummy;
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 
 	if(decoder->state == FLAC__STREAM_DECODER_END_OF_STREAM)
 		return true;
 
-	assert(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC);
+	FLAC__ASSERT(decoder->state == FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC);
 
 	while(1) {
 		switch(decoder->state) {
@@ -336,14 +373,14 @@ bool FLAC__stream_decoder_process_remaining_frames(FLAC__StreamDecoder *decoder)
 			case FLAC__STREAM_DECODER_END_OF_STREAM:
 				return true;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 	}
 }
 
 unsigned FLAC__stream_decoder_input_bytes_unconsumed(FLAC__StreamDecoder *decoder)
 {
-	assert(decoder != 0);
+	FLAC__ASSERT(decoder != 0);
 	return decoder->guts->input.bytes - decoder->guts->input.consumed_bytes;
 }
 
@@ -396,7 +433,7 @@ bool stream_decoder_find_metadata_(FLAC__StreamDecoder *decoder)
 	unsigned i, id;
 	bool first = true;
 
-	assert(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
+	FLAC__ASSERT(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
 
 	for(i = id = 0; i < 4; ) {
 		if(decoder->guts->cached) {
@@ -453,8 +490,9 @@ bool stream_decoder_find_metadata_(FLAC__StreamDecoder *decoder)
 bool stream_decoder_read_metadata_(FLAC__StreamDecoder *decoder)
 {
 	uint32 i, x, last_block, type, length;
+	uint64 xx;
 
-	assert(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
+	FLAC__ASSERT(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
 
 	if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &last_block, FLAC__STREAM_METADATA_IS_LAST_LEN, read_callback_, decoder))
 		return false; /* the read_callback_ sets the state for us */
@@ -464,66 +502,100 @@ bool stream_decoder_read_metadata_(FLAC__StreamDecoder *decoder)
 		return false; /* the read_callback_ sets the state for us */
 	if(type == FLAC__METADATA_TYPE_STREAMINFO) {
 		unsigned used_bits = 0;
-		decoder->guts->stream_header.type = type;
-		decoder->guts->stream_header.is_last = last_block;
-		decoder->guts->stream_header.length = length;
+		decoder->guts->stream_info.type = type;
+		decoder->guts->stream_info.is_last = last_block;
+		decoder->guts->stream_info.length = length;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_MIN_BLOCK_SIZE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.min_blocksize = x;
+		decoder->guts->stream_info.data.stream_info.min_blocksize = x;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_MIN_BLOCK_SIZE_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_MAX_BLOCK_SIZE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.max_blocksize = x;
+		decoder->guts->stream_info.data.stream_info.max_blocksize = x;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_MAX_BLOCK_SIZE_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_MIN_FRAME_SIZE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.min_framesize = x;
+		decoder->guts->stream_info.data.stream_info.min_framesize = x;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_MIN_FRAME_SIZE_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_MAX_FRAME_SIZE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.max_framesize = x;
+		decoder->guts->stream_info.data.stream_info.max_framesize = x;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_MAX_FRAME_SIZE_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_SAMPLE_RATE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.sample_rate = x;
+		decoder->guts->stream_info.data.stream_info.sample_rate = x;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_SAMPLE_RATE_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_CHANNELS_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.channels = x+1;
+		decoder->guts->stream_info.data.stream_info.channels = x+1;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_CHANNELS_LEN;
 
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_STREAMINFO_BITS_PER_SAMPLE_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		decoder->guts->stream_header.data.stream_info.bits_per_sample = x+1;
+		decoder->guts->stream_info.data.stream_info.bits_per_sample = x+1;
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_BITS_PER_SAMPLE_LEN;
 
-		if(!FLAC__bitbuffer_read_raw_uint64(&decoder->guts->input, &decoder->guts->stream_header.data.stream_info.total_samples, FLAC__STREAM_METADATA_STREAMINFO_TOTAL_SAMPLES_LEN, read_callback_, decoder))
+		if(!FLAC__bitbuffer_read_raw_uint64(&decoder->guts->input, &decoder->guts->stream_info.data.stream_info.total_samples, FLAC__STREAM_METADATA_STREAMINFO_TOTAL_SAMPLES_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
 		used_bits += FLAC__STREAM_METADATA_STREAMINFO_TOTAL_SAMPLES_LEN;
 
 		for(i = 0; i < 16; i++) {
 			if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, 8, read_callback_, decoder))
 				return false; /* the read_callback_ sets the state for us */
-			decoder->guts->stream_header.data.stream_info.md5sum[i] = (byte)x;
+			decoder->guts->stream_info.data.stream_info.md5sum[i] = (byte)x;
 		}
 		used_bits += i*8;
 
 		/* skip the rest of the block */
-		assert(used_bits % 8 == 0);
+		FLAC__ASSERT(used_bits % 8 == 0);
 		length -= (used_bits / 8);
 		for(i = 0; i < length; i++) {
 			if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, 8, read_callback_, decoder))
 				return false; /* the read_callback_ sets the state for us */
 		}
 
-		decoder->guts->has_stream_header = true;
-		decoder->guts->metadata_callback(decoder, &decoder->guts->stream_header, decoder->guts->client_data);
+		decoder->guts->has_stream_info = true;
+		decoder->guts->metadata_callback(decoder, &decoder->guts->stream_info, decoder->guts->client_data);
+	}
+	else if(type == FLAC__METADATA_TYPE_SEEKTABLE) {
+		unsigned real_points;
+
+		decoder->guts->seek_table.type = type;
+		decoder->guts->seek_table.is_last = last_block;
+		decoder->guts->seek_table.length = length;
+
+		decoder->guts->seek_table.data.seek_table.num_points = length / FLAC__STREAM_METADATA_SEEKPOINT_LEN;
+
+		if(0 == (decoder->guts->seek_table.data.seek_table.points = (FLAC__StreamMetaData_SeekPoint*)malloc(decoder->guts->seek_table.data.seek_table.num_points * sizeof(FLAC__StreamMetaData_SeekPoint)))) {
+			decoder->state = FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
+			return false;
+		}
+		for(i = real_points = 0; i < decoder->guts->seek_table.data.seek_table.num_points; i++) {
+			if(!FLAC__bitbuffer_read_raw_uint64(&decoder->guts->input, &xx, FLAC__STREAM_METADATA_SEEKPOINT_SAMPLE_NUMBER_LEN, read_callback_, decoder))
+				return false; /* the read_callback_ sets the state for us */
+			decoder->guts->seek_table.data.seek_table.points[real_points].sample_number = xx;
+
+			if(!FLAC__bitbuffer_read_raw_uint64(&decoder->guts->input, &xx, FLAC__STREAM_METADATA_SEEKPOINT_STREAM_OFFSET_LEN, read_callback_, decoder))
+				return false; /* the read_callback_ sets the state for us */
+			decoder->guts->seek_table.data.seek_table.points[real_points].stream_offset = xx;
+
+			if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &x, FLAC__STREAM_METADATA_SEEKPOINT_FRAME_SAMPLES_LEN, read_callback_, decoder))
+				return false; /* the read_callback_ sets the state for us */
+			decoder->guts->seek_table.data.seek_table.points[real_points].frame_samples = x;
+
+			if(decoder->guts->seek_table.data.seek_table.points[real_points].sample_number != FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER)
+				real_points++;
+		}
+		decoder->guts->seek_table.data.seek_table.num_points = real_points;
+
+		decoder->guts->has_seek_table = true;
+		decoder->guts->metadata_callback(decoder, &decoder->guts->seek_table, decoder->guts->client_data);
 	}
 	else {
 		/* skip other metadata blocks */
@@ -570,8 +642,8 @@ bool stream_decoder_frame_sync_(FLAC__StreamDecoder *decoder)
 
 	/* If we know the total number of samples in the stream, stop if we've read that many. */
 	/* This will stop us, for example, from wasting time trying to sync on an ID3V1 tag. */
-	if(decoder->guts->has_stream_header && decoder->guts->stream_header.data.stream_info.total_samples) {
-		if(decoder->guts->samples_decoded >= decoder->guts->stream_header.data.stream_info.total_samples) {
+	if(decoder->guts->has_stream_info && decoder->guts->stream_info.data.stream_info.total_samples) {
+		if(decoder->guts->samples_decoded >= decoder->guts->stream_info.data.stream_info.total_samples) {
 			decoder->state = FLAC__STREAM_DECODER_END_OF_STREAM;
 			return true;
 		}
@@ -650,22 +722,22 @@ bool stream_decoder_read_frame_(FLAC__StreamDecoder *decoder, bool *got_a_frame)
 				/* no adjustment needed */
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_LEFT_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				if(channel == 1)
 					bps++;
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_RIGHT_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				if(channel == 0)
 					bps++;
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_MID_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				if(channel == 1)
 					bps++;
 				break;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 		/*
 		 * now read it
@@ -693,17 +765,17 @@ bool stream_decoder_read_frame_(FLAC__StreamDecoder *decoder, bool *got_a_frame)
 				/* do nothing */
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_LEFT_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				for(i = 0; i < decoder->guts->frame.header.blocksize; i++)
 					decoder->guts->output[1][i] = decoder->guts->output[0][i] - decoder->guts->output[1][i];
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_RIGHT_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				for(i = 0; i < decoder->guts->frame.header.blocksize; i++)
 					decoder->guts->output[0][i] += decoder->guts->output[1][i];
 				break;
 			case FLAC__CHANNEL_ASSIGNMENT_MID_SIDE:
-				assert(decoder->guts->frame.header.channels == 2);
+				FLAC__ASSERT(decoder->guts->frame.header.channels == 2);
 				for(i = 0; i < decoder->guts->frame.header.blocksize; i++) {
 					mid = decoder->guts->output[0][i];
 					side = decoder->guts->output[1][i];
@@ -717,7 +789,7 @@ bool stream_decoder_read_frame_(FLAC__StreamDecoder *decoder, bool *got_a_frame)
 				}
 				break;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 				break;
 		}
 	}
@@ -738,9 +810,10 @@ bool stream_decoder_read_frame_(FLAC__StreamDecoder *decoder, bool *got_a_frame)
 	decoder->sample_rate = decoder->guts->frame.header.sample_rate;
 	decoder->blocksize = decoder->guts->frame.header.blocksize;
 
-	decoder->guts->samples_decoded += decoder->guts->frame.header.blocksize;
+	decoder->guts->samples_decoded = decoder->guts->frame.header.number.sample_number + decoder->guts->frame.header.blocksize;
 
 	/* write it */
+	/* NOTE: some versions of GCC can't figure out const-ness right and will give you an 'incompatible pointer type' warning on arg 3 here: */
 	if(decoder->guts->write_callback(decoder, &decoder->guts->frame, decoder->guts->output, decoder->guts->client_data) != FLAC__STREAM_DECODER_WRITE_CONTINUE)
 		return false;
 
@@ -756,8 +829,10 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 	byte crc8, raw_header[16]; /* MAGIC NUMBER based on the maximum frame header size, including CRC */
 	unsigned raw_header_len;
 	bool is_unparseable = false;
+	const bool is_known_variable_blocksize_stream = (decoder->guts->has_stream_info && decoder->guts->stream_info.data.stream_info.min_blocksize != decoder->guts->stream_info.data.stream_info.max_blocksize);
+	const bool is_known_fixed_blocksize_stream = (decoder->guts->has_stream_info && decoder->guts->stream_info.data.stream_info.min_blocksize == decoder->guts->stream_info.data.stream_info.max_blocksize);
 
-	assert(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
+	FLAC__ASSERT(decoder->guts->input.consumed_bits == 0); /* make sure we're byte aligned */
 
 	/* init the raw header with the saved bits from synchronization */
 	raw_header[0] = decoder->guts->header_warmup[0];
@@ -796,8 +871,8 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 
 	switch(x = raw_header[2] >> 4) {
 		case 0:
-			if(decoder->guts->has_stream_header && decoder->guts->stream_header.data.stream_info.min_blocksize == decoder->guts->stream_header.data.stream_info.max_blocksize) /* i.e. it's a fixed-blocksize stream */
-				decoder->guts->frame.header.blocksize = decoder->guts->stream_header.data.stream_info.min_blocksize;
+			if(is_known_fixed_blocksize_stream)
+				decoder->guts->frame.header.blocksize = decoder->guts->stream_info.data.stream_info.min_blocksize;
 			else
 				is_unparseable = true;
 			break;
@@ -825,14 +900,14 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 			decoder->guts->frame.header.blocksize = 256 << (x-8);
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 			break;
 	}
 
 	switch(x = raw_header[2] & 0x0f) {
 		case 0:
-			if(decoder->guts->has_stream_header)
-				decoder->guts->frame.header.sample_rate = decoder->guts->stream_header.data.stream_info.sample_rate;
+			if(decoder->guts->has_stream_info)
+				decoder->guts->frame.header.sample_rate = decoder->guts->stream_info.data.stream_info.sample_rate;
 			else
 				is_unparseable = true;
 			break;
@@ -875,7 +950,7 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 			decoder->state = FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC;
 			return true;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 	}
 
 	x = (unsigned)(raw_header[3] >> 4);
@@ -903,8 +978,8 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 
 	switch(x = (unsigned)(raw_header[3] & 0x0e) >> 1) {
 		case 0:
-			if(decoder->guts->has_stream_header)
-				decoder->guts->frame.header.bits_per_sample = decoder->guts->stream_header.data.stream_info.bits_per_sample;
+			if(decoder->guts->has_stream_info)
+				decoder->guts->frame.header.bits_per_sample = decoder->guts->stream_info.data.stream_info.bits_per_sample;
 			else
 				is_unparseable = true;
 			break;
@@ -928,7 +1003,7 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 			is_unparseable = true;
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 			break;
 	}
 
@@ -938,7 +1013,7 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 		return true;
 	}
 
-	if(blocksize_hint) {
+	if(blocksize_hint && is_known_variable_blocksize_stream) {
 		if(!FLAC__bitbuffer_read_utf8_uint64(&decoder->guts->input, &xx, read_callback_, decoder, raw_header, &raw_header_len))
 			return false; /* the read_callback_ sets the state for us */
 		if(xx == 0xffffffffffffffff) { /* i.e. non-UTF8 code... */
@@ -948,10 +1023,7 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 			decoder->state = FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC;
 			return true;
 		}
-		if(decoder->guts->has_stream_header && decoder->guts->stream_header.data.stream_info.min_blocksize == decoder->guts->stream_header.data.stream_info.max_blocksize) /* i.e. it's a fixed-blocksize stream */
-			decoder->guts->frame.header.number.sample_number = (uint64)decoder->guts->last_frame_number * (int64)decoder->guts->stream_header.data.stream_info.min_blocksize + xx;
-		else
-			decoder->guts->frame.header.number.sample_number = xx;
+		decoder->guts->frame.header.number.sample_number = xx;
 	}
 	else {
 		if(!FLAC__bitbuffer_read_utf8_uint32(&decoder->guts->input, &x, read_callback_, decoder, raw_header, &raw_header_len))
@@ -964,8 +1036,8 @@ bool stream_decoder_read_frame_header_(FLAC__StreamDecoder *decoder)
 			return true;
 		}
 		decoder->guts->last_frame_number = x;
-		if(decoder->guts->has_stream_header) {
-			decoder->guts->frame.header.number.sample_number = (int64)decoder->guts->stream_header.data.stream_info.min_blocksize * (int64)x;
+		if(decoder->guts->has_stream_info) {
+			decoder->guts->frame.header.number.sample_number = (int64)decoder->guts->stream_info.data.stream_info.min_blocksize * (int64)x;
 		}
 		else {
 			is_unparseable = true;
@@ -1150,7 +1222,7 @@ bool stream_decoder_read_subframe_fixed_(FLAC__StreamDecoder *decoder, unsigned
 				return false;
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 	}
 
 	/* decode the subframe */
@@ -1182,7 +1254,7 @@ bool stream_decoder_read_subframe_lpc_(FLAC__StreamDecoder *decoder, unsigned ch
 	/* read qlp coeff precision */
 	if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &u32, FLAC__SUBFRAME_LPC_QLP_COEFF_PRECISION_LEN, read_callback_, decoder))
 		return false; /* the read_callback_ sets the state for us */
-	if(u32 == 15) {
+	if(u32 == (1u << FLAC__SUBFRAME_LPC_QLP_COEFF_PRECISION_LEN) - 1) {
 		decoder->guts->error_callback(decoder, FLAC__STREAM_DECODER_ERROR_LOST_SYNC, decoder->guts->client_data);
 		decoder->state = FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC;
 		return true;
@@ -1223,12 +1295,15 @@ bool stream_decoder_read_subframe_lpc_(FLAC__StreamDecoder *decoder, unsigned ch
 				return false;
 			break;
 		default:
-			assert(0);
+			FLAC__ASSERT(0);
 	}
 
 	/* decode the subframe */
 	memcpy(decoder->guts->output[channel], subframe->warmup, sizeof(int32) * order);
-	FLAC__lpc_restore_signal(decoder->guts->residual[channel], decoder->guts->frame.header.blocksize-order, subframe->qlp_coeff, order, subframe->quantization_level, decoder->guts->output[channel]+order);
+	if(bps <= 16 && subframe->qlp_coeff_precision <= 16)
+		decoder->guts->local_lpc_restore_signal_16bit(decoder->guts->residual[channel], decoder->guts->frame.header.blocksize-order, subframe->qlp_coeff, order, subframe->quantization_level, decoder->guts->output[channel]+order);
+	else
+		decoder->guts->local_lpc_restore_signal(decoder->guts->residual[channel], decoder->guts->frame.header.blocksize-order, subframe->qlp_coeff, order, subframe->quantization_level, decoder->guts->output[channel]+order);
 
 	return true;
 }
@@ -1267,15 +1342,26 @@ bool stream_decoder_read_residual_partitioned_rice_(FLAC__StreamDecoder *decoder
 	for(partition = 0; partition < partitions; partition++) {
 		if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &rice_parameter, FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_PARAMETER_LEN, read_callback_, decoder))
 			return false; /* the read_callback_ sets the state for us */
-		for(u = (partition_order == 0 || partition > 0)? 0 : predictor_order; u < partition_samples; u++, sample++) {
-#ifdef SYMMETRIC_RICE
-			if(!FLAC__bitbuffer_read_symmetric_rice_signed(&decoder->guts->input, &i, rice_parameter, read_callback_, decoder))
-				return false; /* the read_callback_ sets the state for us */
+		if(rice_parameter < FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_ESCAPE_PARAMETER) {
+			for(u = (partition_order == 0 || partition > 0)? 0 : predictor_order; u < partition_samples; u++, sample++) {
+#ifdef FLAC__SYMMETRIC_RICE
+				if(!FLAC__bitbuffer_read_symmetric_rice_signed(&decoder->guts->input, &i, rice_parameter, read_callback_, decoder))
+					return false; /* the read_callback_ sets the state for us */
 #else
-			if(!FLAC__bitbuffer_read_rice_signed(&decoder->guts->input, &i, rice_parameter, read_callback_, decoder))
-				return false; /* the read_callback_ sets the state for us */
+				if(!FLAC__bitbuffer_read_rice_signed(&decoder->guts->input, &i, rice_parameter, read_callback_, decoder))
+					return false; /* the read_callback_ sets the state for us */
 #endif
-			residual[sample] = i;
+				residual[sample] = i;
+			}
+		}
+		else {
+			if(!FLAC__bitbuffer_read_raw_uint32(&decoder->guts->input, &rice_parameter, FLAC__ENTROPY_CODING_METHOD_PARTITIONED_RICE_RAW_LEN, read_callback_, decoder))
+				return false; /* the read_callback_ sets the state for us */
+			for(u = (partition_order == 0 || partition > 0)? 0 : predictor_order; u < partition_samples; u++, sample++) {
+				if(!FLAC__bitbuffer_read_raw_int32(&decoder->guts->input, &i, rice_parameter, read_callback_, decoder))
+					return false; /* the read_callback_ sets the state for us */
+				residual[sample] = i;
+			}
 		}
 	}
 
