@@ -22,7 +22,6 @@
  * duplicated here.  Look for 'DUPLICATE:' in comments.
  */
 
-#include <assert.h>
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -34,8 +33,10 @@ static const char *sync_string_ = "fLaC"; /* DUPLICATE:FLAC__STREAM_SYNC_STRING
 static const char *metadata_type_string_[] = { /* DUPLICATE:FLAC__MetaDataTypeString */
 	"STREAMINFO",
 	"PADDING",
-	"APPLICATION"
+	"APPLICATION",
+	"SEEKTABLE"
 };
+static const unsigned SEEKPOINT_LEN_ = 18; /* DUPLICATE:FLAC__STREAM_METADATA_SEEKPOINT_LEN */
 
 static int usage(const char *message, ...);
 static bool list(FILE *f, bool verbose);
@@ -128,7 +129,7 @@ int usage(const char *message, ...)
 bool list(FILE *f, bool verbose)
 {
 	byte buf[65536];
-	byte *b;
+	byte *b = buf;
 	FLAC__StreamMetaData metadata;
 	unsigned blocknum = 0, byte_offset = 0, i;
 
@@ -153,7 +154,7 @@ bool list(FILE *f, bool verbose)
 		/* print header */
 		printf("METADATA block #%u\n", blocknum);
 		printf("byte offset: %u\n", byte_offset);
-		printf("type: %u (%s)\n", (unsigned)metadata.type, metadata.type<=FLAC__METADATA_TYPE_APPLICATION? metadata_type_string_[metadata.type] : "UNKNOWN");
+		printf("type: %u (%s)\n", (unsigned)metadata.type, metadata.type<=FLAC__METADATA_TYPE_SEEKTABLE? metadata_type_string_[metadata.type] : "UNKNOWN");
 		printf("is last: %s\n", metadata.is_last? "true":"false");
 		printf("length: %u\n", metadata.length);
 
@@ -189,6 +190,10 @@ bool list(FILE *f, bool verbose)
 				memcpy(buf, metadata.data.application.id, 4);
 				metadata.data.application.data = buf+4;
 				break;
+			case FLAC__METADATA_TYPE_SEEKTABLE:
+				metadata.data.seek_table.num_points = metadata.length / SEEKPOINT_LEN_;
+				b = buf; /* we leave the points in buf for printing later */
+				break;
 			default:
 				printf("SKIPPING block of unknown type\n\n");
 				continue;
@@ -198,7 +203,7 @@ bool list(FILE *f, bool verbose)
 		switch(metadata.type) {
 			case FLAC__METADATA_TYPE_STREAMINFO:
 				printf("minumum blocksize: %u samples\n", metadata.data.stream_info.min_blocksize);
-				printf("maxumum blocksize: %u samples\n", metadata.data.stream_info.max_blocksize);
+				printf("maximum blocksize: %u samples\n", metadata.data.stream_info.max_blocksize);
 				printf("minimum framesize: %u bytes\n", metadata.data.stream_info.min_framesize);
 				printf("maximum framesize: %u bytes\n", metadata.data.stream_info.max_framesize);
 				printf("sample_rate: %u Hz\n", metadata.data.stream_info.sample_rate);
@@ -226,8 +231,14 @@ bool list(FILE *f, bool verbose)
 					hexdump(metadata.data.application.data, metadata.length);
 				}
 				break;
+			case FLAC__METADATA_TYPE_SEEKTABLE:
+				printf("seek points: %u\n", metadata.data.seek_table.num_points);
+				for(i = 0; i < metadata.data.seek_table.num_points; i++, b += SEEKPOINT_LEN_) {
+					printf("\tpoint %d: sample_number=%llu, stream_offset=%llu, frame_samples=%u\n", i, unpack_uint64(b, 8), unpack_uint64(b+8, 8), unpack_uint32(b+16, 2));
+				}
+				break;
 			default:
-				assert(0);
+				FLAC__ASSERT(0);
 		}
 
 		if(!metadata.is_last)
