@@ -20,7 +20,6 @@
 #include <mmreg.h>
 #include <msacm.h>
 #include <math.h>
-#include <assert.h>
 
 #include "in2.h"
 #include "FLAC/all.h"
@@ -395,7 +394,7 @@ void metadata_callback(const FLAC__FileDecoder *decoder, const FLAC__StreamMetaD
 	stream_info_struct *stream_info = (stream_info_struct *)client_data;
 	(void)decoder;
 	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO) {
-		assert(metadata->data.stream_info.total_samples < 0x100000000); /* this plugin can only handle < 4 gigasamples */
+		FLAC__ASSERT(metadata->data.stream_info.total_samples < 0x100000000); /* this plugin can only handle < 4 gigasamples */
 		stream_info->total_samples = (unsigned)(metadata->data.stream_info.total_samples&0xffffffff);
 		stream_info->bits_per_sample = metadata->data.stream_info.bits_per_sample;
 		stream_info->channels = metadata->data.stream_info.channels;
